def log_invocation(fn='/tmp/reflex_invoc.log'):
    """ log invocation of actor for easy reexecution with:
        eval $(cat /tmp/reflex_invoc.log)
    """
    import os, sys
    #if 'REFLEX_DEBUG' not in os.environ:
    #    return
    try:
        with open(fn, 'w') as f:
            path = ""
            s = " ".join(['"%s"' % x for x in sys.argv])
            try:
                import reflex
                path = '"%s:"' % os.path.dirname(reflex.__file__)
            except ImportError:
                pass
            f.write('PYTHONPATH=%s$PYTHONPATH python %s\n' % (path, s)) 
    except:
        pass

# import the needed modules
try:
  import reflex
  import_sucess = True
  import reflex_plot_widgets
  import pipeline_product 
  from pipeline_display import ImageDisplay

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, this is the function to modify:
#  readFitsData()                  (from class DataPlotterManager) 
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the title of the window, modify this function:
#  setWindowTitle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:
    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def readFitsData(self, fitsFiles):
      #Control variable to check if the interesting files where at the input
      self.fibff_all_found = False
      self.slitff_found = False

      #Read all the products
      frames = dict()
      for frame in fitsFiles:
        if frame == '' :
          continue
        category = frame.category
        frames[category] = frame
        #print category
      if "FIB_FF_DTC_REDL" in frames and \
         "FIB_FF_DTC_REDU" in frames :  
        self.fibff_all_found = True  

      if "SLIT_FF_DTC_REDL" in frames and \
         "SLIT_FF_DTC_REDU" in frames :  
        self.slitff_found = True  
 
      if self.fibff_all_found == True :
        frame_redl = frames["FIB_FF_DTC_REDL"]
        frame_redu = frames["FIB_FF_DTC_REDU"]

        self.fibreff_redl = pipeline_product.PipelineProduct(frame_redl)
        self.fibreff_redu = pipeline_product.PipelineProduct(frame_redu)
        self.fibreff_next = self.fibreff_redl.all_hdu[0].header['NAXIS3']

      if self.slitff_found == True :
        frame_redl = frames["SLIT_FF_DTC_REDL"]
        frame_redu = frames["SLIT_FF_DTC_REDU"]

        self.slitff_redl = pipeline_product.PipelineProduct(frame_redl)
        self.slitff_redu = pipeline_product.PipelineProduct(frame_redu)

        self.wlen = self.slitff_redl.all_hdu[0].header['ESO INS GRAT2 WLEN']
        self.slitff_next = self.slitff_redl.all_hdu[0].header['NAXIS3']
        self.is_simcal = self.slitff_redl.all_hdu[0].header['ESO OCS SIMCAL']

      self.list_slitff_fid = []
      for i in range(self.slitff_next):
         self.list_slitff_fid.append(i)

      self.list_fibff_fid = []
      for i in range(self.fibreff_next):
         self.list_fibff_fid.append(i)

      #read first all input image to save time later in case of update 
      self.list_slitff_redl = []
      self.list_slitff_redu = []
      self.list_slitff_redl=self.readAllImages(self.slitff_redl,self.slitff_next)
      self.list_slitff_redu=self.readAllImages(self.slitff_redu,self.slitff_next)

      self.list_fibff_redl = []
      self.list_fibff_redu = []
      self.list_fibff_redl=self.readAllImages(self.fibreff_redl,self.fibreff_next)
      self.list_fibff_redu=self.readAllImages(self.fibreff_redu,self.fibreff_next)


    def plotWidgets(self) :
        widgets = list()

        # Files Selector radiobutton
        labels_fib_ff = self.list_fibff_fid
        labels_slit_ff = self.list_slitff_fid

        self.radiobutton1 = reflex_plot_widgets.InteractiveRadioButtons(self.fibre_flat_selector, self.setFibreSelectCallBack, labels_fib_ff, 0, 
                title="")
        self.radiobutton2 = reflex_plot_widgets.InteractiveRadioButtons(self.slit_flat_selector, self.setSlitSelectCallBack, labels_slit_ff, 0, 
                title="")
        widgets.append(self.radiobutton1)
        widgets.append(self.radiobutton2)

        return widgets

    def setFibreSelectCallBack(self, ext_id) :
        ext_id = int(ext_id)
        self.subplot_allff_redu.cla()
        self.subplot_allff_redl.cla()

        self.displayFibreImages(ext_id)

    def setSlitSelectCallBack(self, ext_id) :
        ext_id = int(ext_id)
        self.subplot_slitff_redu.cla()
        self.subplot_slitff_redl.cla()

        self.displaySlitImages(ext_id)

    def paragraph(self,text, width=None):
       """ wrap text string into paragraph
           text:  text to format, removes leading space and newlines
           width: if not None, wraps text, not recommended for tooltips as
                  they are wrapped by wxWidgets by default
       """
       import textwrap
       #print 'width=',width,'Text=',text
       if width is None:
           return textwrap.dedent(text).replace('\n', ' ').strip()
       else:
           return textwrap.fill(textwrap.dedent(text), width=width) 

    #Read all images
    def readAllImages(self,obj,next):
        list = []
        for i in range(next):
          obj.readImage(fits_extension=0)
          obj.read2DLinearWCS(fits_extension=0)
          list.append(obj.image[i])
        return list

    #2D image display
    def plotImage(self,obj,subplot,title,tooltip,ext_id):
          #The following commands are done by readAllImages()
          #before calling plotImage in an initial call to save time
          #in case the user ask to update the plots interactively
          #
          #obj.readImage(fits_extension=0)
          #obj.read2DLinearWCS(fits_extension=0)
          #print obj.image[ext_id].shape

          img_obj = ImageDisplay()
          image = obj.image[ext_id]
          y_size, x_size = image.shape

	  img_obj.setLabels('X [pix]','Y [pix]')
          img_obj.setXLinearWCSAxis(obj.crval1,obj.cdelt1,obj.crpix1)
          img_obj.setYLinearWCSAxis(obj.crval2,obj.cdelt2,obj.crpix2)

          img_obj.display(subplot, title, self.paragraph(tooltip),image)

    def displayFibreImages(self,fid):
        tooltip=self.setWindowFibreHelp()
        tit_redl=self.setWindowFibreTitle("REDL")
        tit_redu=self.setWindowFibreTitle("REDL")

        self.plotImage(self.fibreff_redu,self.subplot_allff_redl,tit_redu,tooltip,fid)
        self.plotImage(self.fibreff_redl,self.subplot_allff_redu,tit_redl,tooltip,fid)

    def displaySlitImages(self,fid):
        tooltip=self.setWindowSlitHelp()
        tit_redl=self.setWindowSlitTitle("REDL")
        tit_redu=self.setWindowSlitTitle("REDU")

        self.plotImage(self.slitff_redu,self.subplot_slitff_redu,tit_redu,tooltip,fid)
        self.plotImage(self.slitff_redl,self.subplot_slitff_redl,tit_redl,tooltip,fid)


    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
      if self.fibff_all_found == True:

        ncols=2
        nraws=2
        self.subplot_allff_redu  = figure.add_subplot(nraws,ncols,1)
        self.subplot_slitff_redu = figure.add_subplot(nraws,ncols,2)
        self.subplot_allff_redl  = figure.add_subplot(nraws,ncols,3)
        self.subplot_slitff_redl = figure.add_subplot(nraws,ncols,4)

      else : 
        self.subtext_nodata    = figure.add_subplot(1,1,1)

      self.fibre_flat_selector = figure.add_axes([0.10, 0.45, 0.11, 0.10])
      self.slit_flat_selector = figure.add_axes([0.60, 0.45, 0.11, 0.10])
      tip="Frame Id:\n (Mouse \n left\n button)"
      self.fibre_flat_selector.text(0.34, 0.50, tip, color='black', fontsize=12, ha='left', va='center', alpha=1.0)
      self.slit_flat_selector.text(0.34, 0.50, tip, color='black', fontsize=12, ha='left', va='center', alpha=1.0)

    def displayDataNotFound(self):
      #Data not found info
      self.subtext_nodata.set_axis_off()
      self.text_nodata = 'Line prediction not found in the products (PRO.CATG=FIB_FF_DTC_REDL or FIB_FF_DTC_REDU or SLIT_FF_DTC_REDL or SLIT_FF_DTC_REDU)' 
      self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
                                 ha='left', va='center', alpha=1.0)
      self.subtext_nodata.tooltip='Images not found in the products'


    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self):
      if self.fibff_all_found == True :
     
         #self.imageTooltips()
         slitff_id=self.list_slitff_fid[0]
         fibreff_id=self.list_fibff_fid[0]
         self.displayFibreImages(fibreff_id)
         self.displaySlitImages(slitff_id)
  

    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'
    def setInteractiveParameters(self):
      paramList = list()
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_prep_sff_ofpos',displayName='ext_method',group='ofpos',description='Extraction method. <std | opt | fst | fop>'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_prep_sff_ofpos',displayName='filter_switch',group='ofpos',description='Filter switch. <none | median>'))
      paramList.append(reflex.RecipeParameter('flames_cal_prep_sff_ofpos',displayName='sat_thr',group='ofpos',description='Saturation threshold'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_prep_sff_ofpos',displayName='fileprep',group='ofpos',description='Slitff* and Fibreff* file preparation. If fast extraction method is used it should be set to FALSE.'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_prep_sff_ofpos',displayName='save_flat_size',group='ofpos',description='To be sure to use the flat part of a slit flatsone may need to subtract this bit. The default value -1, is used for automatic setting: if WCEN=520 save_flat_size=0, else save_flat_size=2. Values explicitly set by user overwrite this rule. [-1]'))

      return paramList

    def setWindowFibreHelp(self):
      help_text = """
In this window, the user may check the quality of the synthetic fibre
      all-flat which is critical to determine the extraction quality
      of science spectra. The fibres should appear smooth without any
      artefacts. The save_flat_size parameter value may affect quality."""
      return help_text

    def setWindowFibreTitle(self,chip):
      title = 'FLAMES-UVES all fibre flat '+chip
      return title


    def setWindowSlitHelp(self):
      help_text = """
In this window, the user may check the quality of the echelle-slit flats which is critical to determine the extraction quality of science spectra. The images should appear smooth without any artefacts. The save_flat_size parameter value may affect quality."""
      return help_text

    def setWindowSlitTitle(self,chip):
      title = 'FLAMES-UVES slit flat '+chip
      return title

except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"

#This is the 'main' function
if __name__ == '__main__':
  log_invocation()

  # import reflex modules
  import reflex_interactive_app
  import sys

  # import UVES reflex modules
  import flames_plot_common

  # Create interactive application
  interactive_app = reflex_interactive_app.PipelineInteractiveApp(enable_init_sop=True)

  #Check if import failed or not
  if import_sucess == 'false' :
    interactive_app.setEnableGUI('false')

  #Open the interactive window if enabled
  if interactive_app.isGUIEnabled() :
    #Get the specific functions for this window
    dataPlotManager = DataPlotterManager()
    interactive_app.setPlotManager(dataPlotManager)
    interactive_app.showGUI()
  else :
    interactive_app.passProductsThrough()

  # print outputs
  interactive_app.print_outputs()

  sys.exit()




def log_invocation(fn='/tmp/reflex_invoc.log'):
    """ log invocation of actor for easy reexecution with:
        eval $(cat /tmp/reflex_invoc.log)
    """
    import os, sys
    #if 'REFLEX_DEBUG' not in os.environ:
    #    return
    try:
        with open(fn, 'w') as f:
            path = ""
            s = " ".join(['"%s"' % x for x in sys.argv])
            try:
                import reflex
                path = '"%s:"' % os.path.dirname(reflex.__file__)
            except ImportError:
                pass
            f.write('PYTHONPATH=%s$PYTHONPATH python %s\n' % (path, s)) 
    except:
        pass

# import the needed modules
try:
  import reflex
  try:
    from astropy.io import fits as pyfits
  except ImportError:
    import pyfits
  import_sucess = True
  import reflex_plot_widgets
  import pipeline_product

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, this is the function to modify:
#  readFitsData()                  (from class DataPlotterManager) 
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the title of the window, modify this function:
#  setWindowTitle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:
    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def readFitsData(self, fitsFiles):
      #Control variable to check if the interesting files where at the input
      self.line_table_mos_found = False

      #Read all the products
      frames = dict()
      for frame in fitsFiles:
        if frame == '' :
          continue
        category = frame.category
        frames[category] = frame

      if "FIB_LINE_TABLE_REDL" in frames and \
         "FIB_LINE_TABLE_REDU" in frames :  
        self.line_table_mos_found = True  

      if self.line_table_mos_found == True :
        frame_redl = frames["FIB_LINE_TABLE_REDL"]
        frame_redu = frames["FIB_LINE_TABLE_REDU"]
        self.next=len(pyfits.open(frame_redl.name))
        self.nfibres = (self.next-1) // 3
        nf=(self.next-1) // 3
        table = pipeline_product.PipelineProduct(frame_redl)
        wlen = table.all_hdu[0].header['ESO INS GRAT2 WLEN']
        is_simcal = table.all_hdu[0].header['ESO OCS SIMCAL']
        print 'wlen=',wlen
        print "is_simcal=",is_simcal
        if wlen == 520 :
           fibre_min=2
        if wlen == 580 :
           if is_simcal == 1 :
             fibre_min=1
           else :
             fibre_min=2

        if wlen == 860 :
           fibre_min=2

        self.wlen=wlen
        self.fibre_min=fibre_min
        self.list_used_fibres = []
        self.list_line_tab = []
        self.list_objects = []

        for fib_id in range(fibre_min,nf+fibre_min) :
           fib_cnt=fib_id-fibre_min
           self.list_used_fibres.append(fib_id)
           ext_id=fib_cnt*3+1
           self.list_line_tab.append(flames_plot_common.PlotableMosLineTable(frame_redl, frame_redu,ext_id,fib_id,fib_cnt))        
        self.list_objects = []

           #print "Process fibre", fib_id, "out of", nf, "use data from ext id=", ext_id




    def plotWidgets(self) :
        widgets = list()

        # Files Selector radiobutton
        labels = self.list_used_fibres
        self.radiobutton = reflex_plot_widgets.InteractiveRadioButtons(self.fibre_selector, self.setFibreSelectCallBack, labels, 0, 
                title="Fibre id\n (Mouse \n left \n button)")
        widgets.append(self.radiobutton)

        return widgets

    def setFibreSelectCallBack(self, fibre_id) :
        fibre_id = int(fibre_id)
        self.subplot_res_wave.cla()
        self.subplot_res_order.cla()
        self.subplot_fwhm_wave.cla()
        self.subplot_y_x.cla()
        self.subtext_lineinfo.cla()

        self.displayTables(fibre_id-self.fibre_min)

    def displayTables(self,fibre_id):

        #Residuals vs wave
        self.plotScatterRESvsWAVE(fibre_id)

        #Residuals vs order
        if self.line_table_mos_found == True :
           self.plotScatterRESvsORDER(fibre_id)

           #FWHM vs wavelenth
           if self.line_table_mos_found == True :
              self.plotScatterFWHMvsWAVE(fibre_id)

              #Y vs X
              self.plotScatterYvsX(fibre_id)

              #Fith subpanel: a text box
              self.list_line_tab[fibre_id].plotLinesText(self.subtext_lineinfo)
        
           else :
              #Data not found info
              displayDataNotFound()

    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
      if self.line_table_mos_found == True:

        ncols=2
        nraws=3
        self.subplot_res_wave  = figure.add_subplot(nraws,ncols,1)
        self.subplot_res_order = figure.add_subplot(nraws,ncols,2)
        self.subplot_fwhm_wave = figure.add_subplot(nraws,ncols,3)
        self.subplot_y_x       = figure.add_subplot(nraws,ncols,4)
        self.subtext_lineinfo  = figure.add_subplot(nraws,1,3)
      else : 
        self.subtext_nodata    = figure.add_subplot(1,1,1)

      self.fibre_selector = figure.add_axes([0.89, 0.12, 0.10, 0.15])


    def plotScatterTooltips(self):
      tooltip ="""Plot of the wavelength residual [Ang] as a function of wavelength [Ang] for the wavelength calibration model. 
Blue points represent line identifications which were used in the wavelength solution, and red points represent line identifications that were clipped from the wavelength solution. 
Lines that were detected in the arc frame but not identified in the line catalogue are not plotted. 
The solid and dashed horizontal lines represent the median value and the +-1 sigma range respectively."""
      self.tooltip_res_wave=tooltip

      tooltip ="""Plot of the wavelength residual [Ang] as a function of order for the wavelength calibration model. 
Blue points represent line identifications which were used in the wavelength solution, and red points represent line identifications that were clipped from the wavelength solution. 
Lines that were detected in the arc frame but not identified in the line catalogue are not plotted. 
The solid and dashed horizontal lines represent the median value and the +-1 sigma range respectively."""
      self.tooltip_res_order=tooltip

      tooltip ="""Plot of the line FWHM (pix) as a function of wavelength [Ang] for identified lines that were used in the wavelength solution (blue points) and all remaining detected lines (red points). 
The solid and dashed horizontal lines represent the median value and the +-1 sigma range respectively."""
      self.tooltip_fwhm_wave=tooltip

      tooltip ="""Plot of the measured line Y coordinate (pix) versus the measured line X coordinate (pix) for identified lines that were used in the wavelength solution (blue points) and all remaining detected lines (red points).
This plot shows the distribution of detected arc lines on the wavelength calibration arc frame."""
      self.tooltip_y_x=tooltip


    def plotScatterRESvsWAVE(self,fibre_id):
      print "fibre_id=",fibre_id
      title = 'Wave Res: Mean=%#.3g Med=%#.3g RMS=%#.3g'% (self.list_line_tab[fibre_id].mean_all_residual,self.list_line_tab[fibre_id].median_all_residual,self.list_line_tab[fibre_id].std_all_residual) 
      self.title_res_wave=title
      print "Mean=", self.list_line_tab[fibre_id].mean_all_residual
      self.list_line_tab[fibre_id].plotRESvsWAVE(self.subplot_res_wave, title, self.tooltip_res_wave,fibre_id)


    def plotScatterRESvsORDER(self,fibre_id):
      title = 'Wave Res: Mean=%#.3g Med=%#.3g RMS=%#.3g'% (self.list_line_tab[fibre_id].mean_all_residual,self.list_line_tab[fibre_id].median_all_residual,self.list_line_tab[fibre_id].std_all_residual) 
      self.title_res_order=title

      self.list_line_tab[fibre_id].plotRESvsORDER(self.subplot_res_order, title, self.tooltip_res_order,fibre_id)

    def plotScatterFWHMvsWAVE(self,fibre_id):
      print "fibere id", fibre_id
      title = 'FWHM: Mean=%#.3g Med=%#.3g RMS=%#.3g'% (self.list_line_tab[fibre_id].mean_all_fwhm,self.list_line_tab[fibre_id].median_all_fwhm,self.list_line_tab[fibre_id].std_all_fwhm) 
      self.title_fwhm_wave=title

      self.list_line_tab[fibre_id].plotFWHMvsWAVE(self.subplot_fwhm_wave, title, self.tooltip_fwhm_wave,fibre_id)

    def plotScatterYvsX(self,fibre_id):
      title   = 'Arcline Positions' 
      tooltip ="""Plot of the measured line Y coordinate (pix) versus the measured line X coordinate (pix) for identified lines that were used in the wavelength solution (blue points) and all remaining detected lines (red points).
This plot shows the distribution of detected arc lines on the wavelength calibration arc frame."""
      self.title_y_x=title
      self.list_line_tab[fibre_id].plotYvsX(self.subplot_y_x, title, self.tooltip_y_x)

    def displayDataNotFound(self):
      #Data not found info
      self.subtext_nodata.set_axis_off()
      self.text_nodata = 'Line prediction not found in the products (PRO.CATG=FIB_LINE_TABLE_REDL or FIB_LINE_TABLE_REDU)' 
      self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
                                 ha='left', va='center', alpha=1.0)
      self.subtext_nodata.tooltip='Line tables not found in the products'

    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self):
      if self.line_table_mos_found == True :
     
        if self.line_table_mos_found == True :
          
          self.plotScatterTooltips()
          fibre_id=self.list_used_fibres[0]
          self.displayTables(fibre_id-self.fibre_min)
  
    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'
    def setInteractiveParameters(self):
      paramList = list()
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_wavecal',displayName='range',group='wavecal',description='Width (pix) of search window is 2*range + 1. This parameter is automatically adjusted according to binning'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_wavecal',displayName='minlines',group='wavecal',description='Minimum number of lines to detect. If zero, the default value (1100 for BLUE/REDL chips; 1000 for REDU chip) is used'))
      paramList.append(reflex.RecipeParameter('flames_cal_wavecal',displayName='maxlines',group='wavecal',description='Maximum number of lines to detect. If zero, the default value (1600 for BLUE/REDL chip; 1400 for REDU chip) is used'))
      paramList.append(reflex.RecipeParameter(recipe='uves_cal_wavecal',displayName='alpha',group='wavecal',description='The parameter that controls the distance to the nearest neighbours'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_wavecal',displayName='degree',group='wavecal',description='Degrees of the global 2d dispersion polynomial. If a negative number is specified, the polynomial degrees are automatically selected by starting from (1, 1) and inreasing the degrees as long as the RMS residual decreases significantly'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_wavecal',displayName='tolerance',group='wavecal',description='Tolerance of fit. If positive, the tolerance is in pixel units. If negative, abs(tolerance) is in wavelength units. Lines with residuals worse than the tolerance are excluded from the final fit. Unlike in previous versions, this parameter is not corrected for CCD binning. This rejection based on the absolute residual in pixel can be effectively disabled by setting the tolerance to a very large number (e.g. 9999). In that case outliers will be rejected using only kappa sigma clipping'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_wavecal',displayName='kappa',group='wavecal',description='Lines with residuals more then kappa stdev are rejected from the final fit'))

      return paramList

    def setWindowHelp(self):
      help_text = """
In this window, the user should aim to improve the wavelength calibration solution by minimising the RMS statistic for the wavelength residuals (shown in the titles of the upper two plots) while keeping a reasonable number of emission lines used in the final solution (reported as "Final no. of fitted lines")."""
      return help_text

    def setWindowTitle(self):
      title = 'Uves Interactive Wavelength Calibration'
      return title

except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"

#This is the 'main' function
if __name__ == '__main__':
  log_invocation()

  # import reflex modules
  import reflex_interactive_app
  import sys

  # import UVES reflex modules
  import flames_plot_common

  # Create interactive application
  interactive_app = reflex_interactive_app.PipelineInteractiveApp(enable_init_sop=True)

  #Check if import failed or not
  if import_sucess == 'false' :
    interactive_app.setEnableGUI('false')

  #Open the interactive window if enabled
  if interactive_app.isGUIEnabled() :
    #Get the specific functions for this window
    dataPlotManager = DataPlotterManager()
    interactive_app.setPlotManager(dataPlotManager)
    interactive_app.showGUI()
  else :
    interactive_app.passProductsThrough()

  # print outputs
  interactive_app.print_outputs()

  sys.exit()




try:
  import numpy
  import pipeline_display
  import pipeline_product
except ImportError:
  donothing=1


class Plot:

  def finalSetup(self, subplot, title, tooltip):
    subplot.grid(True)
    subplot.set_title(title, fontsize=11)
    subplot.tooltip = tooltip


class FlamesUvesSpectrum:
  def __init__(self, fits_low, fits_high,fib_id, 
               fits_low_err = None, fits_high_err = None):
    self.spec_low  = pipeline_product.PipelineProduct(fits_low[fib_id])
    self.spec_high = pipeline_product.PipelineProduct(fits_high[fib_id])
    self.spec_low_err = None
    if fits_low_err != None and fits_high_err != None :
      self.spec_low_err  = pipeline_product.PipelineProduct(fits_low_err[fib_id])
      self.spec_high_err = pipeline_product.PipelineProduct(fits_high_err[fib_id])
    self.loadFromFits()
    self.specdisp = pipeline_display.SpectrumDisplay()

  def loadFromFits(self) :
    self.spec_low.readSpectrum()
    self.spec_high.readSpectrum()
    if self.spec_low_err != None :
      self.spec_low_err.readSpectrum()
      self.spec_high_err.readSpectrum()
    #self.qc_ex_nord_low    = self.spec_low.readKeyword('ESO QC EX NORD', 0)
    #self.qc_ex_ysize_low   = self.spec_low.readKeyword('ESO QC EX YSIZE', 0)
    #self.qc_ex_nord_high   = self.spec_high.readKeyword('ESO QC EX NORD', 0)
    #self.qc_ex_ysize_high  = self.spec_high.readKeyword('ESO QC EX YSIZE', 0)

  def plot(self, subplot, title, tooltip):
    self.specdisp.setLabels(self.spec_low.type1,
                           '['+self.spec_low.bunit+']')
    if self.spec_low_err == None :
      self.specdisp.display(subplot, title, tooltip, 
                            self.spec_low.wave, self.spec_low.flux, 
                            autolimits=True)
      (flux_low_plotmin, flux_low_plotmax)   = self.specdisp.flux_lim
      self.specdisp.display(subplot, title, tooltip, 
                            self.spec_high.wave, self.spec_high.flux, 
                            autolimits=True)
      (flux_high_plotmin, flux_high_plotmax) = self.specdisp.flux_lim
    else :
      self.specdisp.display(subplot, title, tooltip, 
                           self.spec_low.wave, 
                           self.spec_low.flux, self.spec_low_err.flux, 
                           autolimits=True)
      (flux_low_plotmin, flux_low_plotmax)   = self.specdisp.flux_lim
      self.specdisp.display(subplot, title, tooltip, 
                           self.spec_high.wave, 
                           self.spec_high.flux, self.spec_high_err.flux, 
                           autolimits=True)
      (flux_high_plotmin, flux_high_plotmax) = self.specdisp.flux_lim
      
    subplot.set_xlim(min(self.spec_low.start_wave, self.spec_high.start_wave),
                     max(self.spec_low.end_wave, self.spec_high.end_wave))
    flux_plotmin           = min(flux_low_plotmin, flux_high_plotmin) 
    flux_plotmin           = min(flux_plotmin, 0) 
    flux_plotmax           = max(flux_low_plotmax, flux_high_plotmax)
    subplot.set_ylim(flux_plotmin,flux_plotmax)





class FlamesUvesSN:
  def __init__(self, fits_low, fits_high,fib_id, 
               fits_low_err, fits_high_err):
    self.spec_low  = pipeline_product.PipelineProduct(fits_low[fib_id])
    self.spec_high = pipeline_product.PipelineProduct(fits_high[fib_id])
    self.spec_low_err  = pipeline_product.PipelineProduct(fits_low_err[fib_id])
    self.spec_high_err = pipeline_product.PipelineProduct(fits_high_err[fib_id])
    self.loadFromFits()
    self.specdisp = pipeline_display.SpectrumDisplay()

  def loadFromFits(self) :
    self.spec_low.readSpectrum()
    self.spec_high.readSpectrum()
    self.spec_low_err.readSpectrum()
    self.spec_high_err.readSpectrum()
  

  def plot(self, subplot, title, tooltip):
    self.specdisp.setLabels(self.spec_low.type1,
                           '['+self.spec_low.bunit+']')
    import numpy as np
    np.save('/tmp/data1.npy', self.spec_low.flux)
    np.save('/tmp/data2.npy', self.spec_low_err.flux)
    self.specdisp.display(subplot, title, tooltip, 
                            self.spec_low.wave, 
                            self.spec_low.flux / self.spec_low_err.flux, 
                            autolimits=True)
    (flux_low_plotmin, flux_low_plotmax)   = self.specdisp.flux_lim
    self.specdisp.display(subplot, title, tooltip, 
                            self.spec_high.wave, 
                            self.spec_high.flux /  self.spec_high_err.flux,
                            autolimits=True)
    (flux_high_plotmin, flux_high_plotmax) = self.specdisp.flux_lim
      
    subplot.set_xlim(min(self.spec_low.start_wave, self.spec_high.start_wave),
                     max(self.spec_low.end_wave, self.spec_high.end_wave))
    flux_plotmin           = min(flux_low_plotmin, flux_high_plotmin) 
    flux_plotmin           = min(flux_plotmin, 0) 
    flux_plotmax           = max(flux_low_plotmax, flux_high_plotmax)
    subplot.set_ylim(flux_plotmin,flux_plotmax)


class ScatterPlot(Plot):
  def __init__(self, fits):
    self.loadFromFits(fits)

  def scatterPlot(self, subplot, x, y, x_label, y_label, title, tooltip):
    self.scatterPlotScatter(subplot, x, y)
    self.setLabels(subplot, x_label, y_label,fontsize=11)
    self.scatterPlotLimits(subplot, x, y)
    self.finalSetup(subplot, title, tooltip)

  def scatterPlotScatter(self, subplot, x, y):
    subplot.scatter(x, y, s=1.4, color='blue')

  def scatterPlotScatterSmall(self, subplot, x, y, color_pt='blue'):
    subplot.scatter(x, y, s=0.5, color=color_pt)

  def setLabels(self, subplot, x_label, y_label, fontsize=11):
    subplot.set_xlabel(x_label,fontsize=fontsize)
    subplot.set_ylabel(y_label,fontsize=fontsize)

  def scatterPlotLimits(self, subplot, x, y):
    subplot.set_xlim(0., numpy.nanmax(x) + 1.)
    subplot.set_ylim(0., 1.05 * numpy.nanmax(y))

class PlotableOrderTable(ScatterPlot):
  def __init__(self, fits):
    self.table = pipeline_product.PipelineProduct(fits)
    self.loadFromFits(self.table)

  def loadFromFits(self, table) :
    self.order_raw      = table.all_hdu[1].data.field('Order')
    self.sn_raw         = table.all_hdu[1].data.field('ObjSnBlzCentre')
    self.fwhm_raw       = table.all_hdu[1].data.field('ObjFwhmAvg')
    self.pos_raw        = table.all_hdu[1].data.field('ObjPosOnSlit')
    self.ripple_raw     = table.all_hdu[1].data.field('Ripple')

  def maxOrder(self):
    return numpy.nanmax(self.order_raw)
  
  def maxSN(self):
    return numpy.nanmax(self.sn_raw)
  
  def minSN(self):
    return numpy.nanmin(self.sn_raw)
  
  def maxFWHM(self):
    return numpy.nanmax(self.fwhm_raw)
  
  def minFWHM(self):
    return numpy.nanmin(self.fwhm_raw)

  def maxRipple(self):
    return numpy.nanmax(self.ripple_raw)
  
  def minRipple(self):
    return numpy.nanmin(self.ripple_raw)
  
  def maxLinepos(self):
    return numpy.nanmax(self.pos_raw)
  
  def minLinepos(self):
    return numpy.nanmin(self.pos_raw)

class PlotableBlueOrderTable(PlotableOrderTable):
  def __init__(self, fits):
    self.table = pipeline_product.PipelineProduct(fits)
    self.loadFromFits(self.table)

  def plotSN(self, subplot, title, tooltip):
    self.scatterPlot(subplot, self.order_raw, self.sn_raw, 'Order', 'S/N', title, tooltip)

  def plotFWHM(self, subplot, title, tooltip):
    self.scatterPlot(subplot, self.order_raw, self.fwhm_raw, 'Order', 'FWHM (pix)', title, tooltip)

  def plotRipple(self, subplot, title, tooltip):
    self.setLabels(subplot, 'Order', 'Ripple')
    self.scatterPlotScatter(subplot, self.order_raw, self.ripple_raw)
    subplot.set_xlim(0., self.maxOrder() + 1.)
    self.finalSetup(subplot, title, tooltip)

  def plotLinepos(self, subplot, title, tooltip):
    self.scatterPlot(subplot, self.order_raw, self.pos_raw, 'Order', 'Slit Position (pix)', title, tooltip)
  
class PlotableRedOrderTable(PlotableOrderTable):
  def __init__(self, fits_low, fits_high):
    self.order_low         = PlotableOrderTable(fits_low)
    self.order_high        = PlotableOrderTable(fits_high)
    self.order_high_shift  = self.order_high.order_raw + self.order_low.maxOrder()
    self.max_total_order   = numpy.nanmax(self.order_high_shift)

  def plotSN(self, subplot, title, tooltip):
    self.plotLowHiTextBottom(subplot)
    self.setLabels(subplot, 'Order', 'S/N')
    self.scatterPlotScatter(subplot, self.order_low.order_raw, self.order_low.sn_raw)
    self.scatterPlotScatter(subplot, self.order_high_shift,    self.order_high.sn_raw)
    subplot.set_xlim(0., self.max_total_order + 1.)
    subplot.set_ylim(0., 1.05 * max(self.order_low.maxSN(), self.order_high.maxSN()))
    self.plotLowHiSepLine(subplot)
    self.finalSetup(subplot, title, tooltip)

  def plotFWHM(self, subplot, title, tooltip):
    self.plotLowHiTextBottom(subplot)
    self.setLabels(subplot, 'Order', 'FWHM (pix)')
    self.scatterPlotScatter(subplot, self.order_low.order_raw, self.order_low.fwhm_raw)
    self.scatterPlotScatter(subplot, self.order_high_shift,    self.order_high.fwhm_raw)
    subplot.set_xlim(0., self.max_total_order + 1.)
    subplot.set_ylim(0., 1.05 * max(self.order_low.maxFWHM(), self.order_high.maxFWHM()))
    self.plotLowHiSepLine(subplot)
    self.finalSetup(subplot, title, tooltip)

  def plotRipple(self, subplot, title, tooltip):
    self.plotLowHiTextBottom(subplot)
    self.setLabels(subplot, 'Order', 'Ripple')
    self.scatterPlotScatter(subplot, self.order_low.order_raw, self.order_low.ripple_raw)
    self.scatterPlotScatter(subplot, self.order_high_shift,    self.order_high.ripple_raw)
    subplot.set_xlim(0., self.max_total_order + 1.)
    self.plotLowHiSepLine(subplot)
    self.finalSetup(subplot, title, tooltip)

  def plotLinepos(self, subplot, title, tooltip):
    self.plotLowHiTextBottom(subplot)
    self.setLabels(subplot, 'Order', 'Slit Position (pix)')
    self.scatterPlotScatter(subplot, self.order_low.order_raw, self.order_low.pos_raw)
    self.scatterPlotScatter(subplot, self.order_high_shift,    self.order_high.pos_raw)
    subplot.set_xlim(0., self.max_total_order + 1.)
    self.plotLowHiSepLine(subplot)
    self.finalSetup(subplot, title, tooltip)

  def plotLowHiSepLine(self, subplot):
    subplot.axvline(linewidth=1.5, color='black', x=self.order_low.maxOrder() + 0.5)

  def plotLowHiTextBottom(self, subplot):
    subplot.text(0.1, 0.1, "RedLo", transform = subplot.transAxes,
                 ha="left",size=13,bbox=dict(boxstyle="round",alpha=0.2))
    subplot.text(0.9, 0.1, "RedHi", transform = subplot.transAxes,
                 ha="right",size=13,bbox=dict(boxstyle="round",alpha=0.2))
    
  def maxSN(self):
    return max(self.order_low.maxSN(), self.order_high.maxSN())
  
  def minSN(self):
    return min(self.order_low.minSN(), self.order_high.minSN())
  
  def maxFWHM(self):
    return max(self.order_low.maxFWHM(), self.order_high.maxFWHM())
  
  def minFWHM(self):
    return min(self.order_low.minFWHM(), self.order_high.minFWHM())

  def maxRipple(self):
    return max(self.order_low.maxRipple(), self.order_high.maxRipple())
  
  def minRipple(self):
    return min(self.order_low.minRipple(), self.order_high.minRipple())
  
  def maxLinepos(self):
    return max(self.order_low.maxLinepos(), self.order_high.maxLinepos())
  
  def minLinepos(self):
    return min(self.order_low.minLinepos(), self.order_high.minLinepos())
  
class PlotableLineGuessTable(ScatterPlot):
  def __init__(self, fits):
    self.table = pipeline_product.PipelineProduct(fits)
    self.loadFromFits(self.table)
  
  def loadFromFits(self, table) :
    self.xdif_raw    = table.all_hdu[1].data.field('XDIF')
    self.ydif_raw    = table.all_hdu[1].data.field('YDIF')
    self.x_raw       = table.all_hdu[1].data.field('X')
    self.ynew_raw    = table.all_hdu[1].data.field('YNEW')
    self.select_raw  = table.all_hdu[1].data.field('SELPLOT')
    self.det_nx      = table.all_hdu[0].header['ESO DET OUT1 NX']
    self.det_ny      = table.all_hdu[0].header['ESO DET OUT1 NY']
    self.xdif_sel    = self.xdif_raw[self.select_raw == 1]
    self.x_sel       = self.x_raw[self.select_raw == 1]
    self.ydif_sel    = self.ydif_raw[self.select_raw == 1]
    self.ynew_sel    = self.ynew_raw[self.select_raw == 1]
    self.min_x       = numpy.nanmin(self.x_raw)
    self.max_x       = numpy.nanmax(self.x_raw)
    self.min_ynew    = numpy.nanmin(self.ynew_raw)
    self.max_ynew    = numpy.nanmax(self.ynew_raw)
    self.median_xdif = numpy.median(self.xdif_sel)
    self.mean_xdif   = numpy.mean(self.xdif_sel)
    self.std_xdif    = numpy.std(self.xdif_sel)
    self.median_ydif = numpy.median(self.ydif_sel)
    self.mean_ydif   = numpy.mean(self.ydif_sel)
    self.std_ydif    = numpy.std(self.ydif_sel)

class PlotableBlueLineGuessTable(PlotableLineGuessTable):

  def plotXDIFvsX(self, subplot, title, tooltip) :
    self.setLabels(subplot, 'X [pix]', 'XDIF [pix]')
    self.scatterPlotScatterSmall(subplot, self.x_sel, self.xdif_sel)
    subplot.set_xlim(self.min_x, self.max_x)
    subplot.axhline(y=self.median_xdif, color='black', lw=1.2)
    subplot.axhline(y=self.median_xdif+self.std_xdif, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_xdif-self.std_xdif, color='black', ls='-.', lw=1.2)
    self.finalSetup(subplot, title, tooltip)

  def plotYDIFvsX(self, subplot, title, tooltip) :
    self.setLabels(subplot, 'X [pix]', 'YDIF [pix]')
    self.scatterPlotScatterSmall(subplot, self.x_sel, self.ydif_sel)
    subplot.set_xlim(self.min_x, self.max_x)
    subplot.axhline(y=self.median_ydif, color='black', lw=1.2)
    subplot.axhline(y=self.median_ydif+self.std_ydif, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_ydif-self.std_ydif, color='black', ls='-.', lw=1.2)
    self.finalSetup(subplot, title, tooltip)

  def plotXDIFvsY(self, subplot, title, tooltip) :
    self.setLabels(subplot,'Y [pix]', 'XDIF [pix]')
    self.scatterPlotScatterSmall(subplot, self.ynew_sel, self.xdif_sel)
    subplot.set_xlim(self.min_ynew, self.max_ynew)
    subplot.axhline(y=self.median_xdif, color='black', lw=1.2)
    subplot.axhline(y=self.median_xdif+self.std_xdif, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_xdif-self.std_xdif, color='black', ls='-.', lw=1.2)
    self.finalSetup(subplot, title, tooltip)

  def plotYDIFvsY(self, subplot, title, tooltip) :
    self.setLabels(subplot,'Y [pix]', 'YDIF [pix]')
    self.scatterPlotScatterSmall(subplot, self.ynew_sel, self.ydif_sel)
    subplot.set_xlim(self.min_ynew, self.max_ynew)
    subplot.axhline(y=self.median_ydif, color='black', lw=1.2)
    subplot.axhline(y=self.median_ydif+self.std_ydif, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_ydif-self.std_ydif, color='black', ls='-.', lw=1.2)
    self.finalSetup(subplot, title, tooltip)

  def plotYDIFvsXDIF(self, subplot, title, tooltip) :
    self.setLabels(subplot,'XDIF [pix]', 'YDIF [pix]')
    self.scatterPlotScatterSmall(subplot, self.xdif_sel, self.ydif_sel)
    subplot.axhline(y=self.median_ydif, color='black', lw=1.2)
    subplot.axvline(x=self.median_xdif, color='black', lw=1.2)
    self.finalSetup(subplot, title, tooltip)

  def plotYvsX(self, subplot, title, tooltip) :
    self.setLabels(subplot,'X [pix]', 'Y [pix]')
    self.scatterPlotScatterSmall(subplot, self.x_raw, self.ynew_raw, 'red')
    self.scatterPlotScatterSmall(subplot, self.x_sel, self.ynew_sel)
    subplot.set_xlim(0., self.det_ny)
    subplot.set_ylim(0., self.det_nx)
    self.finalSetup(subplot, title, tooltip)
 
class PlotableRedLineGuessTable(ScatterPlot):
  def __init__(self, fits_low, fits_high):
    self.tab_low = PlotableLineGuessTable(fits_low)
    self.tab_high = PlotableLineGuessTable(fits_high)
    self.x_sel_shift     = self.tab_high.x_sel + self.tab_low.det_ny
    self.x_raw_shift     = self.tab_high.x_raw + self.tab_low.det_ny
    self.y_sel_shift     = self.tab_high.ynew_sel + self.tab_low.det_nx
    self.y_raw_shift     = self.tab_high.ynew_raw + self.tab_low.det_nx
    self.ynew_sel_shift  = self.tab_high.ynew_sel + self.tab_low.det_nx
    self.median_all_xdif = numpy.median(numpy.concatenate((self.tab_low.xdif_sel, self.tab_high.xdif_sel)))
    self.mean_all_xdif   = numpy.mean(numpy.concatenate((self.tab_low.xdif_sel, self.tab_high.xdif_sel)))
    self.std_all_xdif    = numpy.std(numpy.concatenate((self.tab_low.xdif_sel, self.tab_high.xdif_sel)))
    self.median_all_ydif = numpy.median(numpy.concatenate((self.tab_low.ydif_sel, self.tab_high.ydif_sel)))
    self.mean_all_ydif   = numpy.mean(numpy.concatenate((self.tab_low.ydif_sel, self.tab_high.ydif_sel)))
    self.std_all_ydif    = numpy.std(numpy.concatenate((self.tab_low.ydif_sel, self.tab_high.ydif_sel)))

  def plotGridLowHiTextRight(self, grid):
    grid[1].text(0.9, 0.1, "RedLo", transform = grid[1].transAxes,
                 ha="right",size=13,bbox=dict(boxstyle="round",alpha=0.2))
    grid[0].text(0.9, 0.9, "RedHi", transform = grid[0].transAxes,
                 ha="right",va="top",size=13,bbox=dict(boxstyle="round",alpha=0.2))

  def plotLowHiTextBottom(self, subplot):
    subplot.text(0.1, 0.1, "RedLo", transform = subplot.transAxes,
                 ha="left",size=13,bbox=dict(boxstyle="round",alpha=0.2))
    subplot.text(0.9, 0.1, "RedHi", transform = subplot.transAxes,
                 ha="right",size=13,bbox=dict(boxstyle="round",alpha=0.2))

  def plotLowHiTextRight(self, subplot):
    subplot.text(0.9, 0.1, "RedLo", transform = subplot.transAxes,
                 ha="right",size=13,bbox=dict(boxstyle="round",alpha=0.2))
    subplot.text(0.9, 0.9, "RedHi", transform = subplot.transAxes,
                 ha="right",va="top",size=13,bbox=dict(boxstyle="round",alpha=0.2))

  def plotGridXDIFvsX(self, grid, title, tooltip) :
    self.plotGridLowHiTextRight(grid)
    self.setLabels(grid[0], 'X [pix]', 'XDIF [pix]')
    self.scatterPlotScatterSmall(grid[0], self.tab_high.x_sel, self.tab_high.xdif_sel)
    self.setLabels(grid[1], 'X [pix]', 'XDIF [pix]')
    self.scatterPlotScatterSmall(grid[1], self.tab_low.x_sel, self.tab_low.xdif_sel)
    grid[0].set_xlim(min(self.tab_low.min_x, self.tab_high.min_x), max(self.tab_low.max_x, self.tab_high.max_x))
    grid[1].set_xlim(min(self.tab_low.min_x, self.tab_high.min_x), max(self.tab_low.max_x, self.tab_high.max_x))
#    Should we plot here the median of that chip or the median of all the values?
    grid[0].axhline(y=self.tab_high.median_xdif, color='black', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_xdif, color='black', lw=1.2)
    grid[0].axhline(y=self.tab_high.median_xdif+self.tab_high.std_xdif, color='black', ls='-.', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_xdif+self.tab_low.std_xdif, color='black', ls='-.', lw=1.2)
    grid[0].axhline(y=self.tab_high.median_xdif-self.tab_high.std_xdif, color='black', ls='-.', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_xdif-self.tab_low.std_xdif, color='black', ls='-.', lw=1.2)
    self.finalSetup(grid[0], title, tooltip)
    self.finalSetup(grid[1], "", tooltip)

  def plotXDIFvsX(self, subplot, title, tooltip) :
    self.plotLowHiTextBottom(subplot)
    self.setLabels(subplot, 'X [pix]', 'XDIF [pix]')
    self.scatterPlotScatterSmall(subplot, self.x_sel_shift, self.tab_high.xdif_sel)
    self.scatterPlotScatterSmall(subplot, self.tab_low.x_sel, self.tab_low.xdif_sel)
    subplot.set_xlim(min(self.tab_low.min_x, self.tab_high.min_x + self.tab_low.det_ny), max(self.tab_low.max_x, self.tab_high.max_x + self.tab_low.det_ny))
    subplot.axhline(y=self.median_all_xdif, color='black', lw=1.2)
    subplot.axhline(y=self.median_all_xdif+self.std_all_xdif, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_all_xdif-self.std_all_xdif, color='black', ls='-.', lw=1.2)
    subplot.axvline(linewidth=1.5, color='black', x=self.tab_low.det_ny)
    self.finalSetup(subplot, title, tooltip)

  def plotGridYDIFvsX(self, grid, title, tooltip) :
    self.plotGridLowHiTextRight(grid)
    self.setLabels(grid[0], 'X [pix]', 'YDIF [pix]')
    self.scatterPlotScatterSmall(grid[0], self.tab_high.x_sel, self.tab_high.ydif_sel)
    self.setLabels(grid[1], 'X [pix]', 'YDIF [pix]')
    self.scatterPlotScatterSmall(grid[1], self.tab_low.x_sel, self.tab_low.ydif_sel)
    grid[0].set_xlim(min(self.tab_low.min_x, self.tab_high.min_x), max(self.tab_low.max_x, self.tab_high.max_x))
    grid[1].set_xlim(min(self.tab_low.min_x, self.tab_high.min_x), max(self.tab_low.max_x, self.tab_high.max_x))
#    Should we plot here the median of that chip or the median of all the values?
    grid[0].axhline(y=self.tab_high.median_ydif, color='black', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_ydif, color='black', lw=1.2)
    grid[0].axhline(y=self.tab_high.median_ydif+self.tab_high.std_ydif, color='black', ls='-.', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_ydif+self.tab_low.std_ydif, color='black', ls='-.', lw=1.2)
    grid[0].axhline(y=self.tab_high.median_ydif-self.tab_high.std_ydif, color='black', ls='-.', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_ydif-self.tab_low.std_ydif, color='black', ls='-.', lw=1.2)
    self.finalSetup(grid[0], title, tooltip)
    self.finalSetup(grid[1], "", tooltip)

  def plotYDIFvsX(self, subplot, title, tooltip) :
    self.plotLowHiTextBottom(subplot)
    self.setLabels(subplot, 'X [pix]', 'YDIF [pix]')
    self.scatterPlotScatterSmall(subplot, self.x_sel_shift, self.tab_high.ydif_sel)
    self.scatterPlotScatterSmall(subplot, self.tab_low.x_sel, self.tab_low.ydif_sel)
    subplot.set_xlim(min(self.tab_low.min_x, self.tab_high.min_x + self.tab_low.det_ny), max(self.tab_low.max_x, self.tab_high.max_x + self.tab_low.det_ny))
    subplot.axhline(y=self.median_all_ydif, color='black', lw=1.2)
    subplot.axhline(y=self.median_all_ydif+self.std_all_ydif, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_all_ydif-self.std_all_ydif, color='black', ls='-.', lw=1.2)
    subplot.axvline(linewidth=1.5, color='black', x=self.tab_low.det_ny)
    self.finalSetup(subplot, title, tooltip)

  def plotGridXDIFvsY(self, grid, title, tooltip) :
    self.plotGridLowHiTextRight(grid)
    self.setLabels(grid[0], 'Y [pix]', 'XDIF [pix]')
    self.scatterPlotScatterSmall(grid[0], self.tab_high.ynew_sel, self.tab_high.xdif_sel)
    self.setLabels(grid[1], 'Y [pix]', 'XDIF [pix]')
    self.scatterPlotScatterSmall(grid[1], self.tab_low.ynew_sel, self.tab_low.xdif_sel)
    grid[0].set_xlim(min(self.tab_low.min_ynew, self.tab_high.min_ynew), max(self.tab_low.max_ynew, self.tab_high.max_ynew))
    grid[1].set_xlim(min(self.tab_low.min_ynew, self.tab_high.min_ynew), max(self.tab_low.max_ynew, self.tab_high.max_ynew))
#    Should we plot here the median of that chip or the median of all the values?
    grid[0].axhline(y=self.tab_high.median_xdif, color='black', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_xdif, color='black', lw=1.2)
    grid[0].axhline(y=self.tab_high.median_xdif+self.tab_high.std_xdif, color='black', ls='-.', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_xdif+self.tab_low.std_xdif, color='black', ls='-.', lw=1.2)
    grid[0].axhline(y=self.tab_high.median_xdif-self.tab_high.std_xdif, color='black', ls='-.', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_xdif-self.tab_low.std_xdif, color='black', ls='-.', lw=1.2)
    self.finalSetup(grid[0], title, tooltip)
    self.finalSetup(grid[1], "", tooltip)

  def plotXDIFvsY(self, subplot, title, tooltip) :
    self.plotLowHiTextBottom(subplot)
    self.scatterPlotScatterSmall(subplot, self.ynew_sel_shift, self.tab_high.xdif_sel)
    self.scatterPlotScatterSmall(subplot, self.tab_low.ynew_sel, self.tab_low.xdif_sel)
    self.setLabels(subplot, 'Y [pix]', 'XDIF [pix]')
    subplot.set_xlim(min(self.tab_low.min_ynew, self.tab_high.min_ynew + self.tab_low.det_nx), max(self.tab_low.max_ynew, self.tab_high.max_ynew + self.tab_low.det_nx))
    subplot.axhline(y=self.median_all_xdif, color='black', lw=1.2)
    subplot.axhline(y=self.median_all_xdif+self.std_all_xdif, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_all_xdif-self.std_all_xdif, color='black', ls='-.', lw=1.2)
    subplot.axvline(linewidth=1.5, color='black', x=self.tab_low.det_nx)
    self.finalSetup(subplot, title, tooltip)

  def plotGridYDIFvsY(self, grid, title, tooltip) :
    self.plotGridLowHiTextRight(grid)
    self.setLabels(grid[0], 'Y [pix]', 'YDIF [pix]')
    self.scatterPlotScatterSmall(grid[0], self.tab_high.ynew_sel, self.tab_high.ydif_sel)
    self.setLabels(grid[1], 'Y [pix]', 'YDIF [pix]')
    self.scatterPlotScatterSmall(grid[1], self.tab_low.ynew_sel, self.tab_low.ydif_sel)
    grid[0].set_xlim(min(self.tab_low.min_ynew, self.tab_high.min_ynew), max(self.tab_low.max_ynew, self.tab_high.max_ynew))
    grid[1].set_xlim(min(self.tab_low.min_ynew, self.tab_high.min_ynew), max(self.tab_low.max_ynew, self.tab_high.max_ynew))
#    Should we plot here the median of that chip or the median of all the values?
    grid[0].axhline(y=self.tab_high.median_ydif, color='black', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_ydif, color='black', lw=1.2)
    grid[0].axhline(y=self.tab_high.median_ydif+self.tab_high.std_ydif, color='black', ls='-.', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_ydif+self.tab_low.std_ydif, color='black', ls='-.', lw=1.2)
    grid[0].axhline(y=self.tab_high.median_ydif-self.tab_high.std_ydif, color='black', ls='-.', lw=1.2)
    grid[1].axhline(y=self.tab_low.median_ydif-self.tab_low.std_ydif, color='black', ls='-.', lw=1.2)
    self.finalSetup(grid[0], title, tooltip)
    self.finalSetup(grid[1], "", tooltip)

  def plotYDIFvsY(self, subplot, title, tooltip) :
    self.plotLowHiTextBottom(subplot)
    self.setLabels(subplot, 'Y [pix]', 'YDIF [pix]')
    self.scatterPlotScatterSmall(subplot, self.ynew_sel_shift, self.tab_high.ydif_sel)
    self.scatterPlotScatterSmall(subplot, self.tab_low.ynew_sel, self.tab_low.ydif_sel)
    subplot.set_xlim(min(self.tab_low.min_ynew, self.tab_high.min_ynew + self.tab_low.det_nx), max(self.tab_low.max_ynew, self.tab_high.max_ynew + self.tab_low.det_nx))
    subplot.axhline(y=self.median_all_ydif, color='black', lw=1.2)
    subplot.axhline(y=self.median_all_ydif+self.std_all_ydif, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_all_ydif-self.std_all_ydif, color='black', ls='-.', lw=1.2)
    subplot.axvline(linewidth=1.5, color='black', x=self.tab_low.det_nx)
    self.finalSetup(subplot, title, tooltip)

  def plotYDIFvsXDIF(self, subplot, title, tooltip) :
    self.setLabels(subplot, 'XDIF [pix]', 'YDIF [pix]')
    self.scatterPlotScatterSmall(subplot, self.tab_high.xdif_sel, self.tab_high.ydif_sel)
    self.scatterPlotScatterSmall(subplot, self.tab_low.xdif_sel, self.tab_low.ydif_sel,'red')
    subplot.axhline(y=self.median_all_ydif, color='black', lw=1.2)
    subplot.axvline(x=self.median_all_xdif, color='black', lw=1.2)
    self.finalSetup(subplot, title, tooltip)

  def plotGridYvsX(self, grid, title, tooltip) :
    self.plotGridLowHiTextRight(grid)
    self.setLabels(grid[0], 'X [pix]', 'Y [pix]')
    self.scatterPlotScatterSmall(grid[0], self.tab_high.x_raw, self.tab_high.ynew_raw, color_pt='red')
    self.scatterPlotScatterSmall(grid[0], self.tab_high.x_sel, self.tab_high.ynew_sel, color_pt='blue')
    self.setLabels(grid[1], 'X [pix]', 'Y [pix]')
    self.scatterPlotScatterSmall(grid[1], self.tab_low.x_raw, self.tab_low.ynew_raw, color_pt='red')
    self.scatterPlotScatterSmall(grid[1], self.tab_low.x_sel, self.tab_low.ynew_sel, color_pt='blue')
    grid[0].set_xlim(0, self.tab_high.det_ny)
    grid[0].set_ylim(0, self.tab_high.det_nx)
    grid[1].set_xlim(0, self.tab_low.det_ny)
    grid[1].set_ylim(0, self.tab_low.det_nx)
    self.finalSetup(grid[0], title, tooltip)
    self.finalSetup(grid[1], "", tooltip)

  def plotYvsX(self, subplot, title, tooltip) :
    self.plotLowHiTextRight(subplot)
    self.setLabels(subplot, 'X [pix]', 'Y [pix]')
    self.scatterPlotScatterSmall(subplot, self.tab_high.x_raw, self.y_raw_shift, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.tab_high.x_sel, self.y_sel_shift, color_pt='blue')
    self.scatterPlotScatterSmall(subplot, self.tab_low.x_raw, self.tab_low.ynew_raw, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.tab_low.x_sel, self.tab_low.ynew_sel, color_pt='blue')
    subplot.set_xlim(0, self.tab_low.det_ny)
    subplot.set_ylim(0, self.tab_high.det_nx + self.tab_low.det_nx)
    subplot.axhline(linewidth=1.5, color='black', y=self.tab_low.det_nx)
    self.finalSetup(subplot, title, tooltip)

class PlotableLineTable(ScatterPlot):
  def __init__(self, fits,ext_id,fib_id):
    self.table = pipeline_product.PipelineProduct(fits)
    self.loadFromFits(self.table,ext_id,fib_id)
  
  def loadFromFits(self, table,ext_id,fib_id) :
    n_win                = (table.all_hdu.__len__() - 1 )//3
    #id_win               = (n_win+1)//2
    #id_win               = (fib_id+1)//2
    #ext_id               = 3 * (id_win -1 ) + 1
    #print 'ext_id=',ext_id,'n_win=',n_win, "fib_id=",fib_id
    self.x_raw           = table.all_hdu[ext_id].data.field('X')
    self.ynew_raw        = table.all_hdu[ext_id].data.field('Ynew')
    self.residual_raw    = table.all_hdu[ext_id].data.field('Residual')
    self.order_raw       = table.all_hdu[ext_id].data.field('Y')
    self.xwidth_raw      = table.all_hdu[ext_id].data.field('Xwidth')
    self.wavec_raw       = table.all_hdu[ext_id].data.field('WaveC')
    self.select_raw      = table.all_hdu[ext_id].data.field('NLinSol')
    self.det_nx          = table.all_hdu[0].header['ESO DET OUT1 NX']
    self.det_ny          = table.all_hdu[0].header['ESO DET OUT1 NY']
    self.nlintot = 0
    self.nlinsel = 0
    self.nlinsol = 0
    self.nlintot         = table.all_hdu[0].header['ESO QC FIB%d NLINTOT' % fib_id]
    self.nlinsel         = table.all_hdu[0].header['ESO QC FIB%d NLINSEL' % fib_id]
    self.nlinsol         = table.all_hdu[0].header['ESO QC FIB%d NLINSOL' % fib_id] 
    self.fwhm            = self.xwidth_raw * 2.35
    self.fwhm_sel        = self.fwhm[self.select_raw == 1]
    self.fwhm_clean      = self.fwhm_sel[numpy.isfinite(self.fwhm_sel)]
    self.x_sel           = self.x_raw[self.select_raw == 1]
    self.ynew_sel        = self.ynew_raw[self.select_raw == 1]
    self.wavec_sel       = self.wavec_raw[self.select_raw == 1]
    self.residual_sel    = self.residual_raw[self.select_raw == 1]
    self.order_sel       = self.order_raw[self.select_raw == 1]
    self.min_wavec       = numpy.nanmin(self.wavec_raw)
    self.max_wavec       = numpy.nanmax(self.wavec_raw)
    self.mean_residual   = numpy.mean(self.residual_sel)
    self.median_residual = numpy.median(self.residual_sel)
    self.std_residual    = numpy.std(self.residual_sel)
    self.max_order       = numpy.nanmax(self.order_raw)
    self.max_fwhm        = numpy.nanmax(self.fwhm)
    self.mean_fwhm       = numpy.mean(self.fwhm_clean)
    self.median_fwhm     = numpy.median(self.fwhm_clean)
    self.std_fwhm        = numpy.std(self.fwhm_clean)

class PlotableBlueLineTable(PlotableLineTable):
  def plotRESvsWAVE(self, subplot, title, tooltip) :
    self.scatterPlotScatterSmall(subplot, self.wavec_raw, self.residual_raw, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.wavec_sel, self.residual_sel)
    self.setLabels(subplot, 'Wavelength [Ang]', 'Wavelength Residual  [Ang]',fontsize=11)

    subplot.set_xlim(self.min_wavec, self.max_wavec)
    subplot.axhline(y=self.median_residual, color='black', lw=1.2)
    subplot.axhline(y=self.median_residual+self.std_residual, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_residual-self.std_residual, color='black', ls='-.', lw=1.2)
    self.finalSetup(subplot, title, tooltip)
    
  def plotRESvsORDER(self, subplot, title, tooltip) :
    self.setLabels(subplot, 'Order', 'Wavelength Residual [Ang]')
    self.scatterPlotScatterSmall(subplot, self.order_raw, self.residual_raw, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.order_sel, self.residual_sel)
    subplot.set_xlim(0., self.max_order + 1.)
    subplot.axhline(y=self.median_residual, color='black', lw=1.2)
    subplot.axhline(y=self.median_residual+self.std_residual, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_residual-self.std_residual, color='black', ls='-.', lw=1.2)
    self.finalSetup(subplot, title, tooltip)
    
  def plotFWHMvsWAVE(self, subplot, title, tooltip) :
    self.setLabels(subplot, 'Wavelength [Ang]', 'Line FWHM [pix]')
    self.scatterPlotScatterSmall(subplot, self.wavec_raw, self.fwhm, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.wavec_sel, self.fwhm_sel)
    subplot.set_xlim(self.min_wavec, self.max_wavec)
    subplot.set_ylim(0., self.max_fwhm * 1.05)
    subplot.axhline(y=self.median_fwhm, color='black', lw=1.2)
    subplot.axhline(y=self.median_fwhm+self.std_fwhm, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_fwhm-self.std_fwhm, color='black', ls='-.', lw=1.2)
    self.finalSetup(subplot, title, tooltip)

  def plotYvsX(self, subplot, title, tooltip) :
    self.setLabels(subplot,'X [pix]', 'Y [pix]')
    self.scatterPlotScatterSmall(subplot, self.x_raw, self.ynew_raw, 'red')
    self.scatterPlotScatterSmall(subplot, self.x_sel, self.ynew_sel)
    subplot.set_xlim(0., self.det_ny)
    subplot.set_ylim(0., self.det_nx)
    self.finalSetup(subplot, title, tooltip)

  def plotLinesText(self, subplot) :
     subplot.set_axis_off()
     text1 = 'No. of detected lines = %g'%self.nlintot 
     subplot.text(0.05, 0.80, text1, color='#11557c', fontsize=15, ha='left', va='center', alpha=1.0)
     text2 = 'Initial no. of fitted lines = %g'%self.nlinsel
     subplot.text(0.05, 0.65, text2, color='#11557c', fontsize=15, ha='left', va='center', alpha=1.0)
     text3 = 'Final no. of fitted lines = %g'%self.nlinsol 
     subplot.text(0.05, 0.50, text3, color='#11557c', fontsize=15, ha='left', va='center', alpha=1.0)
     subplot.tooltip='Lines info'

class PlotableMosLineTable(ScatterPlot):
  def __init__(self, fits_low, fits_high,ext_id=None, fib_id=None, fib_cnt=None):


    #self.min_wavec = []
    #self.max_wavec = []
    #self.median_all_residual = []

    self.tab_low              = PlotableLineTable(fits_low,ext_id,fib_id)
    self.tab_high             = PlotableLineTable(fits_high,ext_id,fib_id)
    #print "flames_plot_common: fib_id=",fib_id,"fib_cnt=",fib_cnt
    self.min_wavec = min(self.tab_low.min_wavec, self.tab_high.min_wavec)
    self.max_wavec = max(self.tab_low.max_wavec, self.tab_high.max_wavec)
    self.median_all_residual = numpy.median(numpy.concatenate((self.tab_low.residual_sel, self.tab_high.residual_sel)))
    self.mean_all_residual    = numpy.mean(numpy.concatenate((self.tab_low.residual_sel, self.tab_high.residual_sel)))
    self.std_all_residual     = numpy.std(numpy.concatenate((self.tab_low.residual_sel, self.tab_high.residual_sel)))
    self.median_all_fwhm      = numpy.median(numpy.concatenate((self.tab_low.fwhm_sel, self.tab_high.fwhm_sel)))
    self.mean_all_fwhm        = numpy.mean(numpy.concatenate((self.tab_low.fwhm_sel, self.tab_high.fwhm_sel)))
    self.std_all_fwhm         = numpy.std(numpy.concatenate((self.tab_low.fwhm_sel, self.tab_high.fwhm_sel)))
    self.order_high_shift     = self.tab_high.order_raw + self.tab_low.max_order
    self.order_high_shift_sel = self.order_high_shift[self.tab_high.select_raw == 1]
    self.max_total_order      = numpy.nanmax(self.order_high_shift)
    self.max_fwhm             = max(self.tab_low.max_fwhm, self.tab_high.max_fwhm)
    self.x_sel_shift          = self.tab_high.x_sel + self.tab_low.det_ny
    self.x_raw_shift          = self.tab_high.x_raw + self.tab_low.det_ny
    self.y_sel_shift          = self.tab_high.ynew_sel + self.tab_low.det_nx
    self.y_raw_shift          = self.tab_high.ynew_raw + self.tab_low.det_nx
    #print "fib id",fib_id,"median",self.median_all_residual
  def plotGridLowHiTextRight(self, grid):
    grid[0].text(0.9, 0.9, "RedHi", transform = grid[0].transAxes,
                 ha="right",va="top",size=13,bbox=dict(boxstyle="round",alpha=0.2))
    grid[1].text(0.9, 0.1, "RedLo", transform = grid[1].transAxes,
                 ha="right",size=13,bbox=dict(boxstyle="round",alpha=0.2))

  def plotLowHiTextBottom(self, subplot):
    subplot.text(0.9, 0.1, "RedHi", transform = subplot.transAxes,
                 ha="right", size=13,bbox=dict(boxstyle="round",alpha=0.2))
    subplot.text(0.1, 0.1, "RedLo", transform = subplot.transAxes,
                 ha="left",size=13,bbox=dict(boxstyle="round",alpha=0.2))

  def plotLowHiTextTop(self, subplot):
    subplot.text(0.9, 0.8, "RedHi", transform = subplot.transAxes,
                 ha="right", size=13,bbox=dict(boxstyle="round",alpha=0.2))
    subplot.text(0.1, 0.8, "RedLo", transform = subplot.transAxes,
                 ha="left",size=13,bbox=dict(boxstyle="round",alpha=0.2))

  def plotLowHiTextRight(self, subplot):
    subplot.text(0.9, 0.1, "RedLo", transform = subplot.transAxes,
                 ha="right",size=13,bbox=dict(boxstyle="round",alpha=0.2))
    subplot.text(0.9, 0.9, "RedHi", transform = subplot.transAxes,
                 ha="right",va="top",size=13,bbox=dict(boxstyle="round",alpha=0.2))

  def plotRESvsWAVE(self, subplot, title, tooltip,fibre_id=None) :
    self.plotLowHiTextTop(subplot)
    self.setLabels(subplot, 'Wavelength [Ang]', 'Wave Resid [Ang]')
    self.scatterPlotScatterSmall(subplot, self.tab_low.wavec_raw, self.tab_low.residual_raw, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.tab_low.wavec_sel, self.tab_low.residual_sel)
    self.scatterPlotScatterSmall(subplot, self.tab_high.wavec_raw, self.tab_high.residual_raw, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.tab_high.wavec_sel, self.tab_high.residual_sel)
    #print "plotRESvsWAVE: fibre_id=",fibre_id
    subplot.set_xlim(self.min_wavec, self.max_wavec)
    subplot.axhline(y=self.median_all_residual, color='black', lw=1.2)
    subplot.axhline(y=self.median_all_residual+self.std_all_residual, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_all_residual-self.std_all_residual, color='black', ls='-.', lw=1.2)
    subplot.axvline(linewidth=1.5, color='black', x=(self.tab_low.max_wavec + self.tab_high.min_wavec)/2.)
    self.finalSetup(subplot, title, tooltip)

  def plotRESvsORDER(self, subplot, title, tooltip,fibre_id=None) :
    self.plotLowHiTextTop(subplot)
    self.setLabels(subplot, 'Order', 'Wave Resid [Ang]')
    self.scatterPlotScatterSmall(subplot, self.tab_low.order_raw, self.tab_low.residual_raw, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.tab_low.order_sel, self.tab_low.residual_sel)
    self.scatterPlotScatterSmall(subplot, self.order_high_shift, self.tab_high.residual_raw, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.order_high_shift_sel, self.tab_high.residual_sel)
    subplot.set_xlim(0., self.max_total_order + 1.)
    subplot.axhline(y=self.median_all_residual, color='black', lw=1.2)
    subplot.axhline(y=self.median_all_residual+self.std_all_residual, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_all_residual-self.std_all_residual, color='black', ls='-.', lw=1.2)
    subplot.axvline(linewidth=1.5, color='black', x=self.tab_low.max_order + 0.5)
    self.finalSetup(subplot, title, tooltip)

  def plotFWHMvsWAVE(self, subplot, title, tooltip,fibre_id=None) :
    self.plotLowHiTextTop(subplot)
    self.setLabels(subplot, 'Wavelength [Ang]', 'Line FWHM [pix]')
    self.scatterPlotScatterSmall(subplot, self.tab_low.wavec_raw, self.tab_low.fwhm, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.tab_low.wavec_sel, self.tab_low.fwhm_sel)
    self.scatterPlotScatterSmall(subplot, self.tab_high.wavec_raw, self.tab_high.fwhm, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.tab_high.wavec_sel, self.tab_high.fwhm_sel)
    subplot.set_xlim(self.min_wavec, self.max_wavec)
    subplot.set_ylim(0., self.max_fwhm * 1.05)
    subplot.axhline(y=self.median_all_fwhm, color='black', lw=1.2)
    subplot.axhline(y=self.median_all_fwhm+self.std_all_fwhm, color='black', ls='-.', lw=1.2)
    subplot.axhline(y=self.median_all_fwhm-self.std_all_fwhm, color='black', ls='-.', lw=1.2)
    subplot.axvline(linewidth=1.5, color='black', x=(self.tab_low.max_wavec + self.tab_high.min_wavec)/2.)
    self.finalSetup(subplot, title, tooltip)

  def plotGridYvsX(self, grid, title, tooltip) :
    self.plotLowHiTextRight(grid)
    self.setLabels(grid[0], 'X [pix]', 'Y [pix]')
    self.scatterPlotScatterSmall(grid[0], self.tab_high.x_raw, self.tab_high.ynew_raw, color_pt='red')
    self.scatterPlotScatterSmall(grid[0], self.tab_high.x_sel, self.tab_high.ynew_sel, color_pt='blue')
    self.setLabels(grid[1], 'X [pix]', 'Y [pix]')
    self.scatterPlotScatterSmall(grid[1], self.tab_low.x_raw, self.tab_low.ynew_raw, color_pt='red')
    self.scatterPlotScatterSmall(grid[1], self.tab_low.x_sel, self.tab_low.ynew_sel, color_pt='blue')
    grid[0].set_xlim(0, self.tab_high.det_ny)
    grid[0].set_ylim(0, self.tab_high.det_nx)
    grid[1].set_xlim(0, self.tab_low.det_ny)
    grid[1].set_ylim(0, self.tab_low.det_nx)
    self.finalSetup(grid[0], title, tooltip)
    self.finalSetup(grid[1], "", tooltip)

  def plotYvsX(self, subplot, title, tooltip) :
    self.plotLowHiTextRight(subplot)
    self.setLabels(subplot, 'X [pix]', 'Y [pix]')
    self.scatterPlotScatterSmall(subplot, self.tab_high.x_raw, self.y_raw_shift, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.tab_high.x_sel, self.y_sel_shift, color_pt='blue')
    self.scatterPlotScatterSmall(subplot, self.tab_low.x_raw, self.tab_low.ynew_raw, color_pt='red')
    self.scatterPlotScatterSmall(subplot, self.tab_low.x_sel, self.tab_low.ynew_sel, color_pt='blue')
    subplot.set_xlim(0, self.tab_low.det_ny)
    subplot.set_ylim(0, self.tab_high.det_nx + self.tab_low.det_nx)
    subplot.axhline(linewidth=1.5, color='black', y=self.tab_low.det_nx)
    self.finalSetup(subplot, title, tooltip)

  def plotLinesText(self, subplot) :
     subplot.set_axis_off()
     text1 = 'No. of detected lines = %g'%(self.tab_low.nlintot + self.tab_high.nlintot)  
     subplot.text(0.05, 0.80, text1, color='#11557c', fontsize=15, ha='left', va='center', alpha=1.0)
     text2 = 'Initial no. of fitted lines = %g'%(self.tab_low.nlinsel + self.tab_high.nlinsel)
     subplot.text(0.05, 0.65, text2, color='#11557c', fontsize=15, ha='left', va='center', alpha=1.0)
     text3 = 'Final no. of fitted lines = %g'%(self.tab_low.nlinsol + self.tab_high.nlinsol) 
     subplot.text(0.05, 0.50, text3, color='#11557c', fontsize=15, ha='left', va='center', alpha=1.0)
     subplot.tooltip='Lines info'




def log_invocation(fn='/tmp/reflex_invoc.log'):
    """ log invocation of actor for easy reexecution with:
        eval $(cat /tmp/reflex_invoc.log)
    """
    import os, sys
    #if 'REFLEX_DEBUG' not in os.environ:
    #    return
    try:
        with open(fn, 'w') as f:
            path = ""
            s = " ".join(['"%s"' % x for x in sys.argv])
            try:
                import reflex
                path = '"%s:"' % os.path.dirname(reflex.__file__)
            except ImportError:
                pass
            f.write('PYTHONPATH=%s$PYTHONPATH python %s\n' % (path, s)) 
    except:
        pass




# import the needed modules
try:
  import matplotlib
  import reflex_plot_widgets
  from pipeline_product import *
  matplotlib.use('WXAgg')
  import_sucess = 'true'

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, this is the function to modify:
#  readFitsData()                  (from class DataPlotterManager) 
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the title of the window, modify this function:
#  setWindowTitle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:
    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def readFitsData(self, fitsFiles):
      #Control variable to ced_heck if the interesting files where at the input
      self.red_sci_point_mos_found = False
      self.sci_info_tab_mos_found = False
      self.cubify = False

      #Read all the products
      from collections import defaultdict
      frames = defaultdict(list)

      list_hdus_sci_redl = list()
      list_hdus_sci_redu = list()
      list_hdus_err_sci_redl = list()
      list_hdus_err_sci_redu = list()
      list_hdus_sci_info_tab_redu = list()
      list_sci_redl = list()
      list_sci_redu = list()
      list_err_sci_redl = list()
      list_err_sci_redu = list()
      list_sci_info_tab_redl = list()
      list_sci_info_tab_redu = list()

      for frame in fitsFiles:
        if frame == '' :
          continue
        pipe_product = PipelineProduct(frame)
        hdulist = pipe_product.hdulist()  
        hdus_item = dict()
        #hdulist = frame.hdulist()
        category = frame.category
        frames[category].append(frame)
        #print category, frame.name

        if category == "MWXB_SCI_REDL" :
           list_sci_redl.append(frame)
           hdus_item[category]=hdulist
           list_hdus_sci_redl.append(hdus_item[category])
           #print category

        elif category == "MWXB_SCI_REDU" :
           list_sci_redu.append(frame)
           hdus_item[category]=hdulist
           list_hdus_sci_redu.append(hdus_item[category])
           #print category

        elif category == "ERR_MWXB_SCI_REDL" :
           list_err_sci_redl.append(frame)
           hdus_item[category]=hdulist
           list_hdus_err_sci_redl.append(hdus_item[category])           
           #print category

        elif category == "ERR_MWXB_SCI_REDU" :
           list_err_sci_redu.append(frame)
           hdus_item[category]=hdulist
           list_hdus_err_sci_redu.append(hdus_item[category])
           #print category

        elif category == "FIB_SCI_INFO_TAB_REDL":
           self.sci_info_tab_mos_found = True  
           list_sci_info_tab_redl.append(frame)
           hdus_item[category]=hdulist
           hdu_sci_info_tab_redl = hdus_item[category]

           #get WLEN and SIMCAL setting
           table = pipeline_product.PipelineProduct(frame)
           self.wlen = table.all_hdu[0].header['ESO INS GRAT2 WLEN']
           self.is_simcal = table.all_hdu[0].header['ESO OCS SIMCAL']

        elif "FIB_SCI_INFO_TAB_REDU" in frames :
           self.sci_info_tab_mos_found = True  
           list_sci_info_tab_redu.append(frame)
           hdus_item[category]=hdulist
           list_hdus_sci_info_tab_redu.append(hdus_item[category])
 
      naxis=list_hdus_sci_redl[0][0].header['NAXIS']
      if  naxis == 2  :
          nfl = list_hdus_sci_redl[0].data.shape[0]
          nfu = list_hdus_sci_redu[0].data.shape[0]
          self.cubify = True
          print "Sci Fibre cube format"
      else :
          nfl = len(list_sci_redl)
          nfu = len(list_sci_redu)
          print "Sci fibre normal format"
      
      #print "nf=", nf , "cubify = ", self.cubify

      # rarely happens that no same fibres are extracted on both
      # REDL/U arms. The following trick allow the python display
      # not to crush
      #nf = min(nfl,nfu)
      self.nfibres =  min(nfl,nfu)
      self.list_spec_raw = []
      self.list_snr_raw = []
      #i=1
  
      # Read fibre info table a first time to know if the fibre is a
      # simultaneous calibration fibre or allocated to object or sky
      self.list_obs_obj_cal = []
      self.list_obs_obj_type = []
      for i in range(9):
         self.list_obs_obj_cal.append(hdu_sci_info_tab_redl[1].data['Retractor_pt1'][i])
         self.list_obs_obj_type.append(hdu_sci_info_tab_redl[1].data['TYPE'][i])

     
      # Read fibre info table a second time to get object id and
      # corresponding RA,DEC only for simultaneous calibration fibre
      # and fibres allocated to object or sky      
      self.list_obs_objects = []
      self.list_obs_obj_ra = []
      self.list_obs_obj_dec = []
      for i in range(9):
         if self.list_obs_obj_cal[i] == "Calibration"  and self.is_simcal == 1 :
           self.list_obs_objects.append(hdu_sci_info_tab_redl[1].data['OBJECT'][i])

           self.list_obs_obj_ra.append(hdu_sci_info_tab_redl[1].data['RA'][i])
           self.list_obs_obj_dec.append(hdu_sci_info_tab_redl[1].data['DEC'][i])
    
         if self.list_obs_obj_type[i] == "U" or self.list_obs_obj_type[i] == "S" :
           self.list_obs_objects.append(hdu_sci_info_tab_redl[1].data['OBJECT'][i])
           self.list_obs_obj_ra.append(hdu_sci_info_tab_redl[1].data['RA'][i])
           self.list_obs_obj_dec.append(hdu_sci_info_tab_redl[1].data['DEC'][i])
          
           
      #print self.list_obs_objects

      self.list_used_fibres = []
      for i in range(self.nfibres):
        #table=list_hdus_sci_info_tab_redl[0].data[i,:]
        self.red_sci_point_mos_found = True
        if self.cubify == True :
           print "cube case"
           #import pdb
           #pdb.set_trace()
           self.list_spec_raw.append(UvesRedSpectrum(list_hdus_sci_redl[0].data[i,:],list_hdus_sci_redu[0].data[i,:],list_hdus_err_sci_redl[0].data[i,:],list_hdus_err_sci_redu[0].data[i,:]))
        else :
           #print "normal case i=",i,self.list_obs_obj_cal[i],self.list_obs_obj_type[i]
           self.list_used_fibres.append(i)

           #print "add to list",  i,self.list_obs_objects[i] 
           self.list_spec_raw.append(FlamesUvesSpectrum(frames["MWXB_SCI_REDL"],
                                                  frames["MWXB_SCI_REDU"],i,
                                                  frames["ERR_MWXB_SCI_REDL"],
                                                  frames["ERR_MWXB_SCI_REDU"]))
           #self.list_snr_raw.append(FlamesUvesSN(frames["MWXB_SCI_REDL"],
           #                                       frames["MWXB_SCI_REDU"],i,
           #                                       frames["ERR_MWXB_SCI_REDL"],
           #                                       frames["ERR_MWXB_SCI_REDU"]))
 
        #print "no of fibres", self.nfibres
        #print "allocated fibres", self.list_used_fibres


    def plotWidgets(self) :
        widgets = list()
        #labels= [0,1,2,3,4,5,6,7]
        labels = self.list_used_fibres
        # Files Selector radiobutton
        self.radiobutton = reflex_plot_widgets.InteractiveRadioButtons(self.fibre_selector, self.setFibreSelectCallBack, labels, 0, 
                title="")
        widgets.append(self.radiobutton)

        return widgets

    def setFibreSelectCallBack(self, fibre_id) :
        fibre_id = int(fibre_id)
        self.list_subplot_spec[0].cla()
        #self.list_subplot_snr[0].cla()
        #print self.list_spec_raw[fibre_id]
        self.plotData(fibre_id)

    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
      #gs = gridspec.GridSpec(3, 1)
      #self.fibre_selector = figure.add_subplot(gs[0,0])
      if self.red_sci_point_mos_found == True :
        self.list_subplot_spec     = []
        self.list_subplot_snr       = []

        nc=1
        nf=self.nfibres
        nraws=2
        #print "nf=",nf
        i=self.list_used_fibres[0]
        #for i in range(nf) :
        self.list_subplot_spec.append(figure.add_subplot(nraws,nc,nc*i+1))
        #self.list_subplot_snr.append(figure.add_subplot(nraws,nc,nc*i+2))

      else : 
        self.subtext_nodata     = figure.add_subplot(1,1,1)
        
      #self.fibre_selector = figure.add_axes([0.85, 0.85, 0.14, 0.14])
      self.fibre_selector = figure.add_axes([0.85, 0.25, 0.14, 0.14])
      tip="Frame Id:\n (Mouse \n left\n button)"
      self.fibre_selector.text(0.35, 0.50, tip, color='black', fontsize=12, ha='left', va='center', alpha=1.0)
    def plotData(self,fibre_id):
        self.setInfoObj(fibre_id)
        self.plotSpectrum(fibre_id)
        #self.plotSNRSpectrum(fibre_id)

    def setInfoCommon(self):
        self.title_common   = 'Extracted and Merged Spectrum (REDL,REDU).\n'

        self.title_snr   = 'Signal to Noise Ratio of Extracted and Merged Spectrum (REDL,REDU)' 
        self.tooltip_spec = """Plot of the extracted and merged spectrum of the science object (blue line) as total flux (ADU) versus wavelength (Ang). 
The +-1 sigma uncertainties are plotted as the light blue region encompassing the object spectrum (and bounded by the light grey spectra). 
Note that this spectrum is not flux calibrated."""

        self.tooltip_snr = """Plot of the Signal to Noise ratio associated to the extracted and merged spectrum of the science object (blue line) as total flux (ADU) versus wavelength (Ang). 
The +-1 sigma uncertainties are plotted as the light blue region encompassing the object spectrum (and bounded by the light grey spectra). 
Note that this spectrum is not flux calibrated."""

    def setInfoSetting(self):
        if self.is_simcal == 1 :
           self.info_setting = 'SimCal %#.3d mode. '% (self.wlen) 
        else :
           self.info_setting = 'OzPoz %#.3d mode. '% (self.wlen) 

    def setInfoObj(self,fibre_id):
        if self.list_obs_obj_cal[fibre_id] == "Calibration" and self.is_simcal == 1 :
           self.info_obj='Simultaneous calibration fibre'
        else :
           self.info_obj='Target: '+self.list_obs_objects[fibre_id]+' RA=%#.4g DEC=%#.4g'% (self.list_obs_obj_ra[fibre_id],self.list_obs_obj_dec[fibre_id])

    def plotSpectrum(self,fibre_id):
        self.title_spec=self.title_common+self.info_setting+self.info_obj
        title=self.title_spec
        tooltip=self.tooltip_spec
        self.list_spec_raw[fibre_id].plot(self.list_subplot_spec[0], title, tooltip)

    def plotSNRSpectrum(self,fibre_id):
        title=self.title_snr
        tooltip=self.tooltip_snr
        self.list_snr_raw[fibre_id].plot(self.list_subplot_snr[0], title, tooltip)

    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self):
      if self.red_sci_point_mos_found == True :
        i=self.list_used_fibres[0] 

        #First subpanel: a spectrum
        if self.red_sci_point_mos_found == True :
           self.setInfoCommon()
           self.setInfoSetting()
           self.setInfoObj(i)
           self.plotData(i)


      else :
        #Data not found info
        self.subtext_nodata.set_axis_off()
        self.text_nodata = """Science object spectra not found in the products:
For Red data:  PRO.CATG=MWXB_SCI_REDL, MWXB_SCI_REDU"""
        self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
                                 ha='left', va='center', alpha=1.0)
        self.subtext_nodata.tooltip='Science object spectra not found in the products'

  
    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'
    def setInteractiveParameters(self):
      paramList = list()
#      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='debug','general','Whether or not to save intermediate results to local directory. [FALSE]'))
#      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='plotter','general',
#      """Any plots produced by the recipe are redirected to the
#         command specified by this parameter. The plotting command must
#         contain the substring 'gnuplot' and must be able to parse
#         gnuplot syntax on its standard input. Valid examples of such
#         a command may include 'gnuplot -persist' and 'cat >
#         mygnuplot$$.gp'. A finer control of the plotting options can
#         be obtained by writing an executable script, e.g. my_gnuplot.pl, that 
#         executes gnuplot after setting the desired gnuplot options
#         (e.g. set terminal pslatex color). To turn off plotting, set
#         this parameter to 'no'. [no]"""))

#      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='process_chip',group='general',"""For
#      RED arm data proces the redl, redu, or both chip(s). <both |
#      redl | redu | REDL | REDU> [both]"""))

      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='ext_method',group='extract',description="""Extraction method. <std | opt | fst | fop> [opt]"""))

#      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='cor_max_fnd',group='extract',description="""Find correlation maximum?. <N | Y> [Y]"""))

      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='cor_max_ rng',group='extract',description="""Correlation range size?. [6.0]"""))

      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='cor_max_ pnt',group='extract',description="""Correlation sampling points?. [25]"""))

      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='cor_def_off',group='extract',description="""Correlation center offset?. [0.0]"""))

      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='corvel_iter',group='extract',description="""Velocity correlation iteration number (SimCal). [1]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='bias_method',group='extract',description="""Bias subtraction method. <M | V | N> [M]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='bias_value',group='extract',description="""Bias value (only if bias_method = V). [200]"""))
#      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='cubify_sw',group='general',description="""Cubify switch. <Y | N> [N]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='filt_sw',group='extract',description="""Filter switch. <none | median> [none]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='bkg_max_io_win',group='background',description="""Background window number in each full inter order. [500]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='bkg_xy_win_sz_x',group='background',description="""x maximum size of each background window:. [6]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='bkg_xy_win_sz_y',group='background',description="""y maximum size of each background window:. [2]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='pixel_thresh_max',group='BP',description="""Pixel saturation threshold max. [55000]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='pixel_thresh_min',group='BP',description="""Pixel saturation threshold min. [-20]""")) 
#     paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='input_fmt_cube',group='general',description="""Input data format. [TRUE]"""))
#     paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='output_fmt_cube',group='general',description="""Output data format. [FALSE]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='drs_k_s_thre',group='general',description="""Kappa sigma threshold. [10.0]"""))
#      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='drs_base_name',group='general',description="""Base name for science products. [fxb]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='drs_maxyshift',group='extract',description="""Half width of the interval to scan for correlation, when determining y shift. [3.0]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='drs_ext_w_siz',group='extract',description="""Integration window size good: 10 (if fibre deconvolution works fine). [10.0]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='rebin.wavestep',group='rebin',description="""The bin size (in w.l.u.) in wavelength space. If negative, a step size of 2/3 * ( average pixel size ) is used. [-1.0]"""))
      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='rebin.scale',group='rebin',
      description="""Whether
      or not to multiply by the factor dx/dlambda (pixels per
      wavelength) during the rebinning. This option is disabled as
      default in concordance with the method used in the MIDAS
      pipeline. This option should be set to true to convert the
      observed flux (in pixel-space) to a flux per wavelength (in 
      wavelength-space). [FALSE]"""))

      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='merge',group='merge',
      description="""Order merging method. If 'optimal', the flux in the
      overlapping region is set to the (optimally computed, using the 
      uncertainties) average of single order spectra. If 'sum', the
      flux in the overlapping region is computed as the sum of the
      single order spectra. If flat-fielding is done, method 'optimal'
      is recommended, otherwise 'sum'. <optimal | sum> [optimal]"""))

      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='merge_delt1',group='merge',
      description="""Order merging left hand (short wavelength) cut. To reduce the
      amount of order overlapping regions we allow to cut short and
      long wavelength ranges. This may reduce the ripple possibly
      introduced by the order merging. Suggested values are: 10 
      (W<=390), 12 (390<W<=437, 520<W<=564), 14 (437<W<=520, 564<W). [0.0]"""))

      paramList.append(RecipeParameter(recipe='flames_obs_scired',displayName='merge_delt2',group='merge',
      description="""Order merging right hand (long wavelength) cut. To reduce the
      amount of order overlapping regions we allow to cut short and
      long wavelength ranges. This may reduce the ripple possibly
      introduced by the order merging. Suggested values is 4. [0.0]"""))


      return paramList

    def setWindowHelp(self):
      help_text = """
In this window, the user should check that the science object extracted spectrum is of a good quality by using the pan and zoom buttons at the top-left of this window. 
Attempt to optimise the S/N of the extracted spectrum as a function of spectral order (the upper plot of the bottom-left plots) by choosing different parameter values and re-running the pipeline recipe."""
      return help_text

    def setWindowTitle(self):
      title = 'Uves Interactive Spectrum Extraction'
      return title


except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"

#This is the 'main' function
if __name__ == '__main__':
  log_invocation()

  # import reflex modules
  from reflex import *
  from reflex_interactive_app import *
  from pipeline_display import *

  # import UVES reflex modules
  from flames_plot_common import *

  # Create interactive application
  interactive_app = PipelineInteractiveApp(enable_init_sop=True)

  #Check if import failed or not
  if import_sucess == 'false' :
    interactive_app.setEnableGUI('false')

  #Open the interactive window if enabled
  if interactive_app.isGUIEnabled() :
    #Get the specific functions for this window
    dataPlotManager = DataPlotterManager()
    interactive_app.setPlotManager(dataPlotManager)
    interactive_app.showGUI()
  else :
    interactive_app.passProductsThrough()

  # print outputs
  interactive_app.print_outputs()

  sys.exit()



# import the needed modules
try:
  import reflex
  import_sucess = True

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, this is the function to modify:
#  readFitsData()                  (from class DataPlotterManager) 
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the title of the window, modify this function:
#  setWindowTitle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:
    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def readFitsData(self, fitsFiles):
      #Control variable to check if the interesting files where at the input
      self.red_sci_point_2d_found = False
      self.red_sci_point_blue_found = False
      self.fluxcal_sci_point_blue_found = False
      self.red_sci_point_red_found = False
      self.fluxcal_sci_point_red_found = False
      otype = "POINT_"
      #Read all the products
      frames = dict()
      for frame in fitsFiles:
        if frame == '' :
          continue
        category = frame.category
        frames[category] = frame

      print frames.keys()
      if [x for x in frames.keys() if x.startswith("RED_SCI_POINT")]:
        otype = "POINT_"
      elif [x for x in frames.keys() if x.startswith("RED_SCI_SLICER")]:
        otype = "SLICER_"
      elif [x for x in frames.keys() if x.startswith("RED_SCI_EXTND")]:
        otype = "EXTND_"
      elif [x for x in frames.keys() if x.startswith("RED_2D_SCIENCE")]: 
        self.red_sci_point_2d_found = True
      elif [x for x in frames.keys() if x.startswith("RED_2D_SCI_POINT")]: 
        self.red_sci_point_2d_found = True
      elif [x for x in frames.keys() if x.startswith("RED_2D_SCI_EXTND")]: 
        self.red_sci_point_2d_found = True

      self.otype=otype
      print otype

      key1 = "RED_SCI_"+otype+"BLUE"
      key2 = "ERRORBAR_SCI_"+otype+"BLUE"
      key3 = "ORDER_EXTRACT_QC_BLUE"
      if key1 in frames and \
         key2 in frames and \
         key3 in frames :
        self.red_sci_point_blue_found = True
        self.spec_raw = uves_plot_common.UvesBlueSpectrum(frames["RED_SCI_"+otype+"BLUE"],
                                         frames["ERRORBAR_SCI_"+otype+"BLUE"])
        self.order_table = uves_plot_common.PlotableBlueOrderTable(frames["ORDER_EXTRACT_QC_BLUE"])

        key4 = "FLUXCAL_SCI_"+otype+"BLUE"
        key5 = "FLUXCAL_ERRORBAR_SCI_"+otype+"BLUE"
        if key4 in frames and \
           key5 in frames : 
          self.fluxcal_sci_point_blue_found = True
          self.spec_fluxcal  = uves_plot_common.UvesBlueSpectrum(frames["FLUXCAL_SCI_"+otype+"BLUE"],
                                                frames["FLUXCAL_ERRORBAR_SCI_"+otype+"BLUE"])
        

      key1 = "RED_SCI_"+otype+"REDL"
      key2 = "RED_SCI_"+otype+"REDU"
      key3 = "ERRORBAR_SCI_"+otype+"REDL"
      key4 = "ERRORBAR_SCI_"+otype+"REDU"
      key5 = "ORDER_EXTRACT_QC_REDL"
      key6 = "ORDER_EXTRACT_QC_REDU"
      if key1 in frames and \
         key2 in frames and \
         key3 in frames and \
         key4 in frames and \
         key5 in frames and \
         key6 in frames : 
        self.red_sci_point_red_found = True
        self.spec_raw  = uves_plot_common.UvesRedSpectrum(frames["RED_SCI_"+otype+"REDL"],
                                         frames["RED_SCI_"+otype+"REDU"],
                                         frames["ERRORBAR_SCI_"+otype+"REDL"],
                                         frames["ERRORBAR_SCI_"+otype+"REDU"])
        self.order_table = uves_plot_common.PlotableRedOrderTable(frames["ORDER_EXTRACT_QC_REDL"],
                                                 frames["ORDER_EXTRACT_QC_REDU"])


        key1 = "FLUXCAL_SCI_"+otype+"REDL"
        key2 = "FLUXCAL_SCI_"+otype+"REDU"
        key3 = "FLUXCAL_ERRORBAR_SCI_"+otype+"REDL"
        key4 = "FLUXCAL_ERRORBAR_SCI_"+otype+"REDU"
        if key1 in frames and \
           key2 in frames and \
           key3 in frames and \
           key4 in frames :
          self.fluxcal_sci_point_red_found = True
          self.spec_fluxcal = uves_plot_common.UvesRedSpectrum(frames["FLUXCAL_SCI_"+otype+"REDL"],
                                              frames["FLUXCAL_SCI_"+otype+"REDU"],
                                              frames["FLUXCAL_ERRORBAR_SCI_"+otype+"REDL"],
                                              frames["FLUXCAL_ERRORBAR_SCI_"+otype+"REDU"])

    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
      if self.red_sci_point_blue_found == True or \
         self.red_sci_point_red_found == True :
        if self.fluxcal_sci_point_blue_found == True or \
           self.fluxcal_sci_point_red_found == True :
          self.subplot_spec     = figure.add_subplot(3,1,1)
          self.subplot_spec_cal = figure.add_subplot(3,1,2)
          self.subplot_sn       = figure.add_subplot(6,2,9)
          self.subplot_fwhm     = figure.add_subplot(6,2,10)
          self.subplot_ripple   = figure.add_subplot(6,2,11)
          self.subplot_linepos  = figure.add_subplot(6,2,12)
        else :
          self.subplot_spec     = figure.add_subplot(3,1,1)
          self.subplot_sn       = figure.add_subplot(3,2,3)
          self.subplot_fwhm     = figure.add_subplot(3,2,4)
          self.subplot_ripple   = figure.add_subplot(3,2,5)
          self.subplot_linepos  = figure.add_subplot(3,2,6)
      else : 
        self.subtext_nodata     = figure.add_subplot(1,1,1)
          
    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self):
      if self.red_sci_point_blue_found == True or self.red_sci_point_red_found == True :
        #First subpanel: a spectrum
        if      self.red_sci_point_blue_found == True :
          title_spec   = 'Extracted and Merged Spectrum. No. Orders: %g   Slit Length (pix): %#.3g'% (self.spec_raw.qc_ex_nord, self.spec_raw.qc_ex_ysize)
        elif self.red_sci_point_red_found == True :
          title_spec   = 'Extracted and Merged Spectrum. RedLo: No. Orders: %g   Slit Length (pix): %#.3g. RedHi: No. Orders: %g   Slit Length (pix): %#.3g'% (self.spec_raw.qc_ex_nord_low, self.spec_raw.qc_ex_ysize_low, self.spec_raw.qc_ex_nord_high, self.spec_raw.qc_ex_ysize_high) 
        tooltip_spec = """Plot of the extracted and merged spectrum of the science object (blue line) as total flux (ADU) versus wavelength (Ang). 
The +-1 sigma uncertainties are plotted as the light blue region encompassing the object spectrum (and bounded by the light grey spectra). 
Note that this spectrum is not flux calibrated."""
        self.spec_raw.plot(self.subplot_spec, title_spec, tooltip_spec)

        if self.fluxcal_sci_point_blue_found == True or self.fluxcal_sci_point_red_found == True :
          if   self.red_sci_point_blue_found == True :
            title_spec_fluxcal = 'Flux Calibrated Spectrum. No. Orders: %g   Slit Length (pix): %#.3g'% (self.spec_raw.qc_ex_nord, self.spec_raw.qc_ex_ysize) 
            tooltip_spec_fluxcal="""Plot of the extracted, merged and flux calibrated spectrum of the science object (blue line) as flux ("""+self.spec_fluxcal.spectrum.bunit+""") versus wavelength (Ang). 
The +-1 sigma uncertainties are plotted as the light blue region encompassing the object spectrum (and bounded by the light grey spectra)."""
          elif self.red_sci_point_red_found == True :
            title_spec_fluxcal = 'Flux Calibrated Spectrum. RedLo: No. Orders: %g   Slit Length (pix): %#.3g. RedHi: No. Orders: %g   Slit Length (pix): %#.3g'% (self.spec_raw.qc_ex_nord_low, self.spec_raw.qc_ex_ysize_low, self.spec_raw.qc_ex_nord_high, self.spec_raw.qc_ex_ysize_high) 
            tooltip_spec_fluxcal="""Plot of the extracted, merged and flux calibrated spectrum of the science object (blue line) as flux ("""+self.spec_fluxcal.spec_low.bunit+""") versus wavelength (Ang). 
The +-1 sigma uncertainties are plotted as the light blue region encompassing the object spectrum (and bounded by the light grey spectra)."""
          self.spec_fluxcal.plot(self.subplot_spec_cal, 
                                 title_spec_fluxcal, tooltip_spec_fluxcal)

        #Second subpanel: a plot
        title_sn   = 'Min S/N: %#.3g   Max S/N: %#.3g'% (self.order_table.minSN(), self.order_table.maxSN()) 
        tooltip_sn = """Plot of the measured S/N of the extracted and merged spectrum as a function of the order."""
        self.order_table.plotSN(self.subplot_sn, title_sn, tooltip_sn)

        #Third subpanel: a plot
        title_fwhm = 'Min FWHM: %#.3g   Max FWHM: %#.3g'% (self.order_table.minFWHM(), self.order_table.maxFWHM()) 
        tooltip_fwhm = """Plot of the spatial FWHM (pix) of the science object spectrum trace as a function of the order."""
        self.order_table.plotFWHM(self.subplot_fwhm, title_fwhm, tooltip_fwhm)

        #Fourth subpanel: a plot
        title_ripple   = 'Min Ripple: %#.3g   Max Ripple: %#.3g'% (self.order_table.minRipple(), self.order_table.maxRipple()) 
        tooltip_ripple = """Plot of the amplitude of systematic ripples in the science spectrum as a function of the order. 
Negative values indicate where the value of this statistic could not be calculated."""
        self.order_table.plotRipple(self.subplot_ripple, title_ripple, tooltip_ripple)

        #Fifth subpanel: a plot
        title_linepos   = 'Min Pos: %#.3g   Max Pos: %#.3g'% (self.order_table.minLinepos(), self.order_table.maxLinepos())
        tooltip_linepos = """Plot of the position along the slit (pix) of the science object spectrum trace as a function of the order."""
        self.order_table.plotLinepos(self.subplot_linepos, title_linepos, tooltip_linepos)
        if      self.red_sci_point_blue_found == True :
          self.subplot_linepos.set_ylim(0., self.spec_raw.qc_ex_ysize)
        elif self.red_sci_point_red_found == True :
          self.subplot_linepos.set_ylim(0., self.spec_raw.qc_ex_ysize_low)

      elif self.red_sci_point_2d_found == True :
        #2D extraction info
        self.subtext_nodata.set_axis_off()
        self.text_nodata = """Science object spectra extracted with method "2D",
Results are not displayed but can be found in the 
reflex_end_products directory.
The user may still change reduction parameters and 
re-run the recipe or continue."""
        self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
                                 ha='left', va='center', alpha=1.0)
        self.subtext_nodata.tooltip='Science object spectra extracted with method 2D'

      else :
        #Data not found info
        self.subtext_nodata.set_axis_off()
        self.text_nodata = """Science object spectra not found in the products:
For Blue data:  PRO.CATG=RED_SCI_%sBLUE
For Red data:  PRO.CATG=RED_SCI_%sREDL, RED_SCI_%sREDU"""%(self.otype,self.otype,self.otype)
        self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
                                 ha='left', va='center', alpha=1.0)
        self.subtext_nodata.tooltip='Science object spectra not found in the products'

  
    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'
    def setInteractiveParameters(self):
      paramList = list()
      paramList.append(reflex.RecipeParameter(recipe='uves_obs_scired',displayName='reduce.extract.method',group='extract',description='Extraction method. (2d/optimal not supported by uves_cal_wavecal, weighted supported only by uves_cal_wavecal). <average | linear | 2d | weighted | optimal>'))
      paramList.append(reflex.RecipeParameter(recipe='uves_obs_scired',displayName='reduce.extract.kappa',group='extract',description='In optimal extraction mode, this is the threshold for bad (i.e. hot/cold) pixel rejection. If a pixel deviates more than kappa*sigma (where sigma is the uncertainty of the pixel flux) from the inferred spatial profile, its weight is set to zero. Range: [-1,100]. If this parameter is negative, no rejection is performed'))
      paramList.append(reflex.RecipeParameter(recipe='uves_obs_scired',displayName='reduce.extract.profile',group='extract',description='In optimal extraction mode, the kind of profile to use. gauss gives a Gaussian profile, moffat gives a Moffat profile with beta=4 and a possible linear sky contribution. virtual uses a virtual resampling algorithm (i.e. measures and uses the actual object profile). constant assumes a constant spatial profile and allows optimal extraction of wavelength calibration frames. auto will automatically select the best method based on the estimated S/N of the object. For low S/N, moffat or gauss are recommended (for robustness). For high S/N, virtual is recommended (for accuracy). In the case of virtual resampling, a precise determination of the order positions is required; therefore the order-definition is repeated using the (assumed non-low S/N) science frame. <constant | gauss | moffat | virtual | auto>'))
      paramList.append(reflex.RecipeParameter(recipe='uves_obs_scired',displayName='reduce.extract.skymethod',group='extract',description='In optimal extraction mode, the sky subtraction method to use. median estimates the sky as the median of pixels along the slit (ignoring pixels close to the object), whereas optimal does a chi square minimization along the slit to obtain the best combined object and sky levels. The optimal method gives the most accurate sky determination but is also a bit slower than the median method. <median | optimal>'))
      paramList.append(reflex.RecipeParameter(recipe='uves_obs_scired',displayName='reduce.extract.oversample',group='extract',description='The oversampling factor used for the virtual resampling algorithm. If negative, the value 5 is used for S/N <=200, and the value 10 is used if the estimated S/N is > 200'))
      paramList.append(reflex.RecipeParameter(recipe='uves_obs_scired',displayName='reduce.slitlength',group='other',description='Extraction slit length (in pixels). If negative, the value inferred from the raw frame header is used'))
      paramList.append(reflex.RecipeParameter(recipe='uves_obs_scired',displayName='reduce.objoffset',group='other',description='Offset (in pixels) of extraction slit with respect to center of order. This parameter applies to linear/average/optimal extraction. For linear/average extraction, if the related parameter objslit is negative, the offset is automatically determined by measuring the actual object position'))
      paramList.append(reflex.RecipeParameter(recipe='uves_obs_scired',displayName='reduce.rebin.wavestep',group='rebinning',description='The bin size (in w.l.u.) in wavelength space. If negative, a step size of 2/3 * ( average pixel size ) is used'))

      return paramList

    def setWindowHelp(self):
      help_text = """
In this window, the user should check that the science object extracted spectrum is of a good quality by using the pan and zoom buttons at the top-left of this window. 
Attempt to optimise the S/N of the extracted spectrum as a function of spectral order (the upper plot of the bottom-left plots) by choosing different parameter values and re-running the pipeline recipe."""
      return help_text

    def setWindowTitle(self):
      title = 'Uves Interactive Spectrum Extraction'
      return title

except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"

#This is the 'main' function
if __name__ == '__main__':

  # import reflex modules
  import reflex_interactive_app
  import sys

  # import UVES reflex modules
  import uves_plot_common

  # Create interactive application
  interactive_app = reflex_interactive_app.PipelineInteractiveApp(enable_init_sop=True)

  #Check if import failed or not
  if import_sucess == 'false' :
    interactive_app.setEnableGUI('false')

  #Open the interactive window if enabled
  if interactive_app.isGUIEnabled() :
    #Get the specific functions for this window
    dataPlotManager = DataPlotterManager()
    interactive_app.setPlotManager(dataPlotManager)
    interactive_app.showGUI()
  else :
    interactive_app.passProductsThrough()

  # print outputs
  interactive_app.print_outputs()

  sys.exit()




# import the needed modules
try:
  import reflex
  import_sucess = True

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, this is the function to modify:
#  readFitsData()                  (from class DataPlotterManager) 
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the title of the window, modify this function:
#  setWindowTitle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:
    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def readFitsData(self, fitsFiles):
      #Control variable to check if the interesting files where at the input
      self.line_guess_tab_mos_found = False
      #Read all the products
      frames = dict()
      for frame in fitsFiles:
        if frame == '' :
          continue
        category = frame.category
        frames[category] = frame

      if "FIB_LIN_GUE_REDL" in frames and \
         "FIB_LIN_GUE_REDU" in frames :  
        self.line_guess_tab_mos_found = True
        frame_redl = frames["FIB_LIN_GUE_REDL"]
        frame_redu = frames["FIB_LIN_GUE_REDU"]
        self.line_guess         = uves_plot_common.PlotableRedLineGuessTable(frame_redl,
                                                            frame_redu)

    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
      if self.line_guess_tab_mos_found == True :
        self.subplot_xdif_x    = figure.add_subplot(3,2,1)
        self.subplot_ydif_x    = figure.add_subplot(3,2,2)
        self.subplot_xdif_y    = figure.add_subplot(3,2,3)
        self.subplot_ydif_y    = figure.add_subplot(3,2,4)
        self.subplot_ydif_xdif = figure.add_subplot(3,2,5)
        self.subplot_y_x       = figure.add_subplot(3,2,6)
      else : 
        self.subtext_nodata    = figure.add_subplot(1,1,1)
          
    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self):
      if self.line_guess_tab_mos_found == True :
        #XDIF vs X plot
        tooltip_xdif_x ="""Plot of the difference between the measured and predicted line X coordinate (pix) versus the measured line X coordinate (pix). 
The solid and dashed horizontal lines represent the median value and the +-1 sigma range respectively."""
        if self.line_guess_tab_mos_found == True:
          title_xdif_x   = 'XDIF: Mean=%#.3g Med=%#.3g RMS=%#.3g'% (self.line_guess.mean_all_xdif,self.line_guess.median_all_xdif,self.line_guess.std_all_xdif) 
        self.line_guess.plotXDIFvsX(self.subplot_xdif_x, title_xdif_x, tooltip_xdif_x)

        #YDIF vs X plot
        tooltip_ydif_x ="""Plot of the difference between the measured and predicted line Y coordinate (pix) versus the measured line X coordinate (pix). 
The solid and dashed horizontal lines represent the median value and the +-1 sigma range respectively."""
        if self.line_guess_tab_mos_found == True:
          title_ydif_x   = 'YDIF: Mean=%#.3g Med=%#.3g RMS=%#.3g'% (self.line_guess.mean_all_ydif,self.line_guess.median_all_ydif,self.line_guess.std_all_ydif) 
        self.line_guess.plotYDIFvsX(self.subplot_ydif_x, title_ydif_x, tooltip_ydif_x)

        #XDIF vs Y plot
        tooltip_xdif_y ="""Plot of the difference between the measured and predicted line X coordinate (pix) versus the measured line Y coordinate (pix). 
The solid and dashed horizontal lines represent the median value and the +-1 sigma range respectively."""
        if self.line_guess_tab_mos_found == True:
          title_xdif_y   = 'XDIF: Mean=%#.3g Med=%#.3g RMS=%#.3g'% (self.line_guess.mean_all_xdif,self.line_guess.median_all_xdif,self.line_guess.std_all_xdif) 
        self.line_guess.plotXDIFvsY(self.subplot_xdif_y, title_xdif_y, tooltip_xdif_y)
      
        #YDIF vs Y plot
        tooltip_ydif_y ="""Plot of the difference between the measured and predicted line Y coordinate (pix) versus the measured line Y coordinate (pix). 
The solid and dashed horizontal lines represent the median value and the +-1 sigma range respectively."""
        if self.line_guess_tab_mos_found == True:
          title_ydif_y   = 'YDIF: Mean=%#.3g Med=%#.3g RMS=%#.3g'% (self.line_guess.mean_all_ydif,self.line_guess.median_all_ydif,self.line_guess.std_all_ydif) 
        self.line_guess.plotYDIFvsY(self.subplot_ydif_y, title_ydif_y, tooltip_ydif_y)
        
        #YDIF vs XDIF plot
        title_ydif_xdif   = '' 
        tooltip_ydif_xdif ="""Plot of the difference between the measured and predicted line Y coordinate (pix) versus the difference between the measured and predicted line X coordinate (pix). Red points refer to REDU arm.
The solid horizontal and vertical lines represent the median values of the differences YDIF and XDIF respectively."""
        self.line_guess.plotYDIFvsXDIF(self.subplot_ydif_xdif, title_ydif_xdif, tooltip_ydif_xdif)

        #Y vs X plot
        title_y_x   = 'Arcline Positions' 
        tooltip_y_x ="""Plot of the measured line Y coordinate (pix) versus the measured line X coordinate (pix).
Blue points represent accepted line identifications, and red points represent unidentified detected lines.  
This plot shows the distribution of detected arc lines on the format check arc frame."""
        self.line_guess.plotYvsX(self.subplot_y_x, title_y_x, tooltip_y_x)

      else :
        #Data not found info
        self.subtext_nodata.set_axis_off()
        self.text_nodata = """Line prediction not found in the products:
For Red data:(PRO.CATG=FIB_LIN_GUE_RED/REDU)"""
        self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', 
                                 fontsize=18, ha='left', va='center', alpha=1.0)
        self.subtext_nodata.tooltip='Line prediction not found in the products'
 
  
    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'
    def setInteractiveParameters(self):
      paramList = list()
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_predict',displayName='mbox_x',group='predict',description='Match box X size'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_predict',displayName='mbox_y',group='predict',description='Match box Y size'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_predict',displayName='trans_x',group='predict',description='Detector translation along X'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_predict',displayName='trans_y',group='predict',description='Detector translation along Y'))
      paramList.append(reflex.RecipeParameter(recipe='flames_cal_predict',displayName='ccd_rot_angle_off',group='predict',description='Offset on CCD rotation angle'))

      return paramList

    def setWindowHelp(self):
      help_text = """
In this window, the user should aim to adjust the median of the XDIF and YDIF values (shown in the titles of the upper two plots) to approximately zero pixels by changing the values of the parameters trans_x and trans_y.
This may be done by adding the current median XDIF and YDIF values to the current trans_x and trans_y parameter values, respectively, and clicking "Re-run recipe", and iterating this procedure until the median XDIF and YDIF values are in the range +-0.1 pix.
Note that the parameter ccd_rot_angle_off may be used to help "tighten" any less-well defined trends in the upper four plots. 
Rotations of the order of 0.01-0.1 in either sense may make improvements."""
      return help_text

    def setWindowTitle(self):
      title = 'Uves Interactive Spectral Format'
      return title

except ImportError:
  import_sucess = False
  print "Error importing modules pyfits, wx, matplotlib, numpy"

#This is the 'main' function
if __name__ == '__main__':

  # import reflex modules
  import reflex_interactive_app
  import sys

  # import UVES reflex modules
  import uves_plot_common

  # Create interactive application
  interactive_app = reflex_interactive_app.PipelineInteractiveApp(enable_init_sop=True)

  #Check if import failed or not
  if import_sucess == False :
    interactive_app.setEnableGUI('false')

  #Open the interactive window if enabled
  if interactive_app.isGUIEnabled() :
    #Get the specific functions for this window
    dataPlotManager = DataPlotterManager()
    interactive_app.setPlotManager(dataPlotManager)
    interactive_app.showGUI()
  else :
    interactive_app.passProductsThrough()

  # print outputs
  interactive_app.print_outputs()

  sys.exit()



# import the needed modules
try:
  import reflex
  import_sucess = True

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, this is the function to modify:
#  readFitsData()                  (from class DataPlotterManager) 
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the title of the window, modify this function:
#  setWindowTitle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:
    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def readFitsData(self, fitsFiles):
      #Control variable to check if the interesting files where at the input
      self.red_std_blue_found = False
      self.instr_response_blue_found = False
      self.red_std_red_found = False
      self.instr_response_red_found = False
      #Read all the products
      frames = dict()
      for frame in fitsFiles:
        if frame == '' :
          continue
        category = frame.category
        frames[category] = frame

      
      if "RED_STD_BLUE" in frames and \
         "ORDER_EXTRACT_QC_BLUE" in frames : 
        self.red_std_blue_found = True
        self.std_raw          = uves_plot_common.UvesBlueSpectrum (frames["RED_STD_BLUE"])
        self.order_table       = uves_plot_common.PlotableBlueOrderTable(frames["ORDER_EXTRACT_QC_BLUE"])
        if "INSTR_RESPONSE_BLUE" in frames : 
          self.instr_response_blue_found = True
          self.resp_raw     = uves_plot_common.UvesBlueSpectrum (frames["INSTR_RESPONSE_BLUE"])

      
      if "RED_STD_REDL" in frames and \
         "RED_STD_REDU" in frames and \
         "ORDER_EXTRACT_QC_REDL" in frames and \
         "ORDER_EXTRACT_QC_REDU" in frames : 
        self.red_std_red_found = True
        self.std_raw     = uves_plot_common.UvesRedSpectrum(frames["RED_STD_REDL"], 
                                           frames["RED_STD_REDU"])
        self.order_table = uves_plot_common.PlotableRedOrderTable(frames["ORDER_EXTRACT_QC_REDL"],
                                                 frames["ORDER_EXTRACT_QC_REDU"])
        if "INSTR_RESPONSE_REDL" in frames and \
           "INSTR_RESPONSE_REDU" in frames : 
          self.instr_response_red_found = True
          self.resp_raw = uves_plot_common.UvesRedSpectrum(frames["INSTR_RESPONSE_REDL"],
                                          frames["INSTR_RESPONSE_REDU"])

    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
      if self.red_std_blue_found  == True or self.red_std_red_found == True:
        self.subplot_std      = figure.add_subplot(3,1,1)
        self.subplot_resp     = figure.add_subplot(3,1,2)
        self.subplot_sn       = figure.add_subplot(6,2,9)
        self.subplot_fwhm     = figure.add_subplot(6,2,10)
        self.subplot_ripple   = figure.add_subplot(6,2,11)
        self.subplot_linepos  = figure.add_subplot(6,2,12)
      else :
        self.subtext_nodata   = figure.add_subplot(1,1,1)

          
    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self):
      if self.red_std_blue_found  == True or self.red_std_red_found  == True:
        #Reduced Spectrum plot
        if self.red_std_blue_found  == True :
          title_std   = 'Extracted and Merged Spectrum. No. Orders: %g   Slit Length (pix): %#.3g'% (self.std_raw.qc_ex_nord, self.std_raw.qc_ex_ysize)
        elif self.red_std_red_found  == True :
          title_std   = 'Extracted and Merged Spectrum. RedLo: No. Orders: %g   Slit Length (pix): %#.3g. RedHi: No. Orders: %g   Slit Length (pix): %#.3g'% (self.std_raw.qc_ex_nord_low, self.std_raw.qc_ex_ysize_low, self.std_raw.qc_ex_nord_high, self.std_raw.qc_ex_ysize_high) 
        tooltip_std ="""Plot of the extracted and merged spectrum of the standard star (blue line) as total flux (ADU) versus wavelength (Ang). 
Note that this spectrum is not flux calibrated."""
        self.std_raw.plot(self.subplot_std, title_std, tooltip_std)

        #Instrument Response plot
        if self.instr_response_blue_found == True or self.instr_response_red_found  == True:
          title_resp   = 'Instrument Response' 
          tooltip_resp ="""Plot of the instrument response versus wavelength (Ang) as obtained by dividing the standard star spectrum from the standard star catalogue by the extracted standard star spectrum, and then smoothing to a low resolution in wavelength."""
          self.resp_raw.plot(self.subplot_resp, title_resp, tooltip_resp)
        else :
          #Info subpanel: a text box
          self.subplot_resp.set_axis_off()
          self.text_resp_not_found1 = 'This standard star has not been found in the standard star calibration database.' 
          self.text_resp_not_found2 = 'Therefore the instrument response has not been calculated.' 
          self.subplot_resp.text(0.00, 0.8, self.text_resp_not_found1, 
                                 color='#11557c', fontsize=18,
                                 ha='left', va='center', alpha=1.0)
          self.subplot_resp.text(0.1, 0.6, self.text_resp_not_found2, 
                                 color='#11557c', fontsize=18,
                                 ha='left', va='center', alpha=1.0)
          self.subplot_resp.tooltip="""This standard star has not been found in the standard star calibration database. 
Therefore the instrument response has not been calculated."""

        #Second subpanel: a plot
        title_sn   = 'Min S/N: %#.3g   Max S/N: %#.3g'% (self.order_table.minSN(), self.order_table.maxSN()) 
        tooltip_sn ="""Plot of the measured S/N of the extracted and merged spectrum as a function of the order."""
        self.order_table.plotSN(self.subplot_sn, title_sn, tooltip_sn)
      
        #Third subpanel: a plot
        title_fwhm   = 'Min FWHM: %#.3g   Max FWHM: %#.3g'% (self.order_table.minFWHM(), self.order_table.maxFWHM()) 
        tooltip_fwhm ="""Plot of the spatial FWHM (pix) of the standard star spectrum trace as a function of the order."""
        self.order_table.plotFWHM(self.subplot_fwhm, title_fwhm, tooltip_fwhm)

        #Fourth subpanel: a plot
        title_ripple   = 'Min Ripple: %#.3g   Max Ripple: %#.3g'% (self.order_table.minRipple(), self.order_table.maxRipple()) 
        tooltip_ripple ="""Plot of the amplitude of systematic ripples in the standard star spectrum as a function of the order.
Negative values indicate where the value of this statistic could not be calculated."""
        self.order_table.plotRipple(self.subplot_ripple, title_ripple, tooltip_ripple)

        #Fifth subpanel: a plot
        title_linepos   = 'Min Pos: %#.3g   Max Pos: %#.3g'% (self.order_table.minLinepos(), self.order_table.maxLinepos()) 
        tooltip_linepos ="""Plot of the position along the slit (pix) of the standard star spectrum trace as a function of the order."""
        self.order_table.plotLinepos(self.subplot_linepos, title_linepos, tooltip_linepos)
        if self.red_std_blue_found  == True :
          self.subplot_linepos.set_ylim(0., self.std_raw.qc_ex_ysize)
        elif self.red_std_red_found  == True :
          self.subplot_linepos.set_ylim(0., self.std_raw.qc_ex_ysize_low)

      else :
        #Data not found info
        self.subtext_nodata.set_axis_off()
        self.text_nodata = """Extracted and merged spectrum for the standard star has not been found in the products:
For Blue data:  PRO.CATG=RED_STD_BLUE
For Red data:  PRO.CATG=RED_STD_REDL, RED_STD_REDU"""
        self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
                                 ha='left', va='center', alpha=1.0)
        self.subtext_nodata.tooltip='Extracted and merged spectrum for the standard star has not been found in the products. Therefore the instrument response has not been calculated.'

  
    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'
    def setInteractiveParameters(self):
      paramList = list()
      paramList.append(reflex.RecipeParameter(recipe='uves_cal_response',displayName='reduce.extract.method',group='extract',description='Extraction method. (2d/optimal not supported by uves_cal_wavecal, weighted supported only by uves_cal_wavecal). <average | linear | 2d | weighted | optimal>'))
      paramList.append(reflex.RecipeParameter(recipe='uves_cal_response',displayName='reduce.extract.kappa',group='extract',description='In optimal extraction mode, this is the threshold for bad (i.e. hot/cold) pixel rejection. If a pixel deviates more than kappa*sigma (where sigma is the uncertainty of the pixel flux) from the inferred spatial profile, its weight is set to zero. Range: [-1,100]. If this parameter is negative, no rejection is performed'))
      paramList.append(reflex.RecipeParameter(recipe='uves_cal_response',displayName='reduce.extract.profile',group='extract',description='In optimal extraction mode, the kind of profile to use. gauss gives a Gaussian profile, moffat gives a Moffat profile with beta=4 and a possible linear sky contribution. virtual uses a virtual resampling algorithm (i.e. measures and uses the actual object profile). constant assumes a constant spatial profile and allows optimal extraction of wavelength calibration frames. auto will automatically select the best method based on the estimated S/N of the object. For low S/N, moffat or gauss are recommended (for robustness). For high S/N, virtual is recommended (for accuracy). In the case of virtual resampling, a precise determination of the order positions is required; therefore the order-definition is repeated using the (assumed non-low S/N) science frame. <constant | gauss | moffat | virtual | auto>'))
      paramList.append(reflex.RecipeParameter(recipe='uves_cal_response',displayName='reduce.extract.skymethod',group='extract',description='In optimal extraction mode, the sky subtraction method to use. median estimates the sky as the median of pixels along the slit (ignoring pixels close to the object), whereas optimal does a chi square minimization along the slit to obtain the best combined object and sky levels. The optimal method gives the most accurate sky determination but is also a bit slower than the median method. <median | optimal>'))
      paramList.append(reflex.RecipeParameter(recipe='uves_cal_response',displayName='reduce.extract.oversample',group='extract',description='The oversampling factor used for the virtual resampling algorithm. If negative, the value 5 is used for S/N <=200, and the value 10 is used if the estimated S/N is > 200'))
      paramList.append(reflex.RecipeParameter(recipe='uves_cal_response',displayName='reduce.slitlength',group='other',description='Extraction slit length (in pixels). If negative, the value inferred from the raw frame header is used'))
      paramList.append(reflex.RecipeParameter(recipe='uves_cal_response',displayName='reduce.objoffset',group='other',description='Offset (in pixels) of extraction slit with respect to center of order. This parameter applies to linear/average/optimal extraction. For linear/average extraction, if the related parameter objslit is negative, the offset is automatically determined by measuring the actual object position'))
      paramList.append(reflex.RecipeParameter(recipe='uves_cal_response',displayName='reduce.rebin.wavestep',group='rebinning',description='The bin size (in w.l.u.) in wavelength space. If negative, a step size of 2/3 * ( average pixel size ) is used'))

      return paramList

    def setWindowHelp(self):
      help_text = """
In this window, the user should check that the standard star extracted spectrum is of a good quality by using the pan and zoom buttons at the top-left of this window.
Optionally try and optimise the S/N of the extracted spectrum as a function of spectral order (the upper plot of the bottom-left plots) by choosing different parameter values and re-running the pipeline recipe."""
      return help_text

    def setWindowTitle(self):
      title = 'Uves Interactive Instrument Response'
      return title

except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"

#This is the 'main' function
if __name__ == '__main__':

  # import reflex modules
  import reflex_interactive_app
  import sys

  # import UVES reflex modules
  import uves_plot_common

  # Create interactive application
  interactive_app = reflex_interactive_app.PipelineInteractiveApp(enable_init_sop=True)

  #Check if import failed or not
  if import_sucess == 'false' :
    interactive_app.setEnableGUI('false')

  #Open the interactive window if enabled
  if interactive_app.isGUIEnabled() :
    #Get the specific functions for this window
    dataPlotManager = DataPlotterManager()
    interactive_app.setPlotManager(dataPlotManager)
    interactive_app.showGUI()
  else :
    interactive_app.passProductsThrough()

  # print outputs
  interactive_app.print_outputs()

  sys.exit()
